#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de socio. Este controlador es utilizado desde las secciones:
# * Sección socio: exportar socio a 812 o a excel


class ExportacionController < ApplicationController
  require 'spreadsheet'
  
  TAMANO_MAXIMUM_COLUMNA = 40
  # en socio: exporta a 812 o excel
  def index
    # Si recibimos un formulario...
    if params[:selector]
      # incialización de las variables
      parametro = ""
      tipo = params[:selector][:tipo]
      params[:selector][:fecha_inicio] = params[:selector]["fecha_inicio(1i)"] + "-" + params[:selector]["fecha_inicio(2i)"] + "-01"
      params[:selector][:fecha_fin] = params[:selector]["fecha_fin(1i)"] + "-" + params[:selector]["fecha_fin(2i)"] + "-01"
      @fecha_inicio = params[:selector][:fecha_inicio]
      @fecha_fin = params[:selector][:fecha_fin]
      @format_color = Spreadsheet::Format.new :pattern => 1, :pattern_fg_color => :aqua
      @no_repetido = Hash.new # el hash que no sirve para no repetir un atributo marcado en una columna

      # En el caso de que se haya escogido una plantilla la usamos, si no generamos una hoja nueva
      # Capturamos los errores en una excepcion por si no es una hoja de calculo buena
      begin
        if params[:selector][:plantilla] && params[:selector][:plantilla] != ""
          libro = Spreadsheet.open Documento.find(params[:selector][:plantilla]).adjunto.path
          hoja = libro.worksheet 0
        else
          libro = Spreadsheet::Workbook.new
          hoja = libro.create_worksheet(:name => tipo) if (tipo == "presupuesto" || tipo == "gasto" || tipo=="transferencia" )
        end
      rescue Ole::Storage::FormatError 
        @export_error = _("ERROR: La plantilla elegida no es una hoja de cálculo válida")
      rescue
        @export_error = _("Error Inesperado leyendo plantilla: Contacte con el administrador del sistema.")
      end

      if libro
        case tipo
          when "matriz" then
            exportar_matriz( libro, {:fecha_inicio => params[:selector][:fecha_inicio], :fecha_fin => params[:selector][:fecha_fin]} ) if params[:seccion] == "proyectos"
          when "presupuesto" then
            exportar_presupuesto_proyecto( hoja, {:fecha_inicio => params[:selector][:fecha_inicio], :fecha_fin => params[:selector][:fecha_fin], :pais => params[:selector][:pais]} ) if params[:seccion] == "proyectos"
            exportar_presupuesto_agente( hoja, {:fecha_inicio => params[:selector][:fecha_inicio], :fecha_fin => params[:selector][:fecha_fin], :pais => params[:selector][:pais]} ) if params[:seccion] == "agentes"
          when "gasto" then
            exportar_gasto_proyecto( hoja, {:fecha_inicio => params[:selector][:fecha_inicio], :fecha_fin => params[:selector][:fecha_fin], :pais => params[:selector][:pais]} ) if params[:seccion] == "proyectos"
            exportar_gasto_agente( hoja, {:fecha_inicio => params[:selector][:fecha_inicio], :fecha_fin => params[:selector][:fecha_fin], :pais => params[:selector][:pais]} ) if params[:seccion] == "agentes"
	  when "transferencia" then
	    exportar_transferencia_proyecto( hoja, {:fecha_inicio => params[:selector][:fecha_inicio], :fecha_fin => params[:selector][:fecha_fin]} ) if params[:seccion] == "proyectos"
        end

        #libro.write fichero
        fichero = Tempfile.new('exportacion.xls')
        libro.write fichero 

        # Comprueba que se haya generado el fichero de salida
         if File.exists?(fichero)
           #flash[:mensaje] = _("Exportación correcta")
           send_file fichero.path,
             :disposition => 'attachment',
             :type => "application/vnd.ms-excel",
             :encoding => 'utf8',
             :filename => "gong_" + tipo + ".xls",
             :stream => false
         else
           @export_error = _("Error en la exportación. Contacte con el administrador del sistema.")
         end
      end
    end
  end

  #--
  # METODOS DE  FORMULARIO
  #++

  # ajax que devuelve un listado de los ficheros de plantilla segun el tipo de exportacion elegido 
  def plantillas
    etiqueta = Etiqueta.find_by_nombre(_("Exportacion Presupuesto")) if params[:selector_tipo] == "presupuesto"
    etiqueta = Etiqueta.find_by_nombre(_("Exportacion Gasto")) if params[:selector_tipo] == "gasto"
    etiqueta = Etiqueta.find_by_nombre(_("Exportacion Transferencia")) if params[:selector_tipo] == "transferencia"
    @plantillas = etiqueta.documento.find(:all,:conditions => [ "proyecto_id = " + @proyecto.id.to_s + " OR proyecto_id IS NULL" ]).collect{ |d| [d.adjunto_file_name, d.id] } if etiqueta
    render :partial => "plantillas", :locals => { :tipo_plantilla => params[:selector_tipo] }
  end


  #--
  # METODOS DE EXPORTACION
  #++

  def exportar_presupuesto_proyecto hoja, parametros={}
    condiciones = "proyecto_id = '" + @proyecto.id.to_s + "'"

    # De momento pasamos de fechas
    #if( parametros )
    #  condiciones << " and fecha_inicio >=  '" + parametros[:fecha_inicio] + "'" unless ( parametros[:fecha_inicio].nil? or parametros[:fecha_inicio].empty?)
    #  condiciones << " and fecha_fin <=  '" + parametros[:fecha_fin] + "'" unless ( parametros[:fecha_fin].nil? or parametros[:fecha_fin].empty?)
    #end

    # Cada campo tiene 1) su rotulo, 2) su atributo (anidado con puntos si hay subatributos. Igual que en el helper
    campos_presupuestos = [	["Id", "id"], ["Agente Nombre", "agente.nombre"], ["Etapa", "etapa.nombre"],
				["Unidad", "unidad"], ["NºUnidades", "numero_unidades"], ["Coste Unitario", "coste_unitario"], ["Importe", "importe"],
				["Moneda Abreviatura", "moneda.abreviatura"], ["Partida Codigo", "partida.codigo"], ["Subpartida", "subpartida.nombre"],
				["Concepto", "concepto"], ["Observaciones", "observaciones"],
				["Actividad", "presupuesto_x_actividad", [[ "Actividad Código","actividad.codigo"], ["Actividad Importe", "importe"], ["Actividad Unidades", "numero_unidades"]]],
				["Financiador", "presupuesto_x_agente", [["Financiador Nombre", "agente.nombre"], ["Financiador Importe", "importe"]]]
				]
    linea_presupuesto = 0
    # PREPARACIÓN DE LA CABEZERA DE PRESUPUESTO
    @tamano_columnas_presupuestos = []
    linea_presupuesto = preparar_cabecera(linea_presupuesto, hoja, campos_presupuestos, "presupuestos")
   
    color = true 
    @presupuestos = Presupuesto.find(:all, :conditions => condiciones).each do |presupuesto|
      linea_presupuesto = inscribir_datos(linea_presupuesto, presupuesto, campos_presupuestos, hoja, @tamano_columnas_presupuestos, color)
      color = !color
    end
  end

  def exportar_presupuesto_agente hoja, parametros={}
    condiciones = "agente_id = '" + @agente.id.to_s + "'"

    # De momento pasamos de fechas
    #if( parametros )
    #  condiciones << " and fecha_inicio >=  '" + parametros[:fecha_inicio] + "'" unless ( parametros[:fecha_inicio].nil? or parametros[:fecha_inicio].empty?)
    #  condiciones << " and fecha_fin <=  '" + parametros[:fecha_fin] + "'" unless ( parametros[:fecha_fin].nil? or parametros[:fecha_fin].empty?)
    #end

    # Cada campo tiene 1) su rotulo, 2) su atributo (anidado con puntos si hay subatributos. Igual que en el helper
    campos_presupuestos = [     ["Id", "id"], ["Proyecto", "proyecto.nombre"], ["Etapa", "etapa.nombre"],
                                ["Unidad", "unidad"], ["NºUnidades", "numero_unidades"], ["Coste Unitario", "coste_unitario"], ["Importe", "importe"],
                                ["Moneda Abreviatura", "moneda.abreviatura"], ["Partida Codigo", "partida.codigo"], ["Subpartida", "subpartida.nombre"],
                                ["Concepto", "concepto"], ["Observaciones", "observaciones"],
                                ]
    linea_presupuesto = 0
    # PREPARACIÓN DE LA CABEZERA DE PRESUPUESTO
    @tamano_columnas_presupuestos = []
    linea_presupuesto = preparar_cabecera(linea_presupuesto, hoja, campos_presupuestos, "presupuestos")

    color = true
    @presupuestos = Presupuesto.find(:all, :conditions => condiciones).each do |presupuesto|
      linea_presupuesto = inscribir_datos(linea_presupuesto, presupuesto, campos_presupuestos, hoja, @tamano_columnas_presupuestos, color)
      color = !color
    end
  end

  def exportar_gasto_proyecto hoja, parametros={}
    condiciones = "gasto_x_proyecto.proyecto_id = '" + @proyecto.id.to_s + "' and gasto_x_proyecto.porcentaje = '1'"

    # De momento pasamos de fechas
    #if( parametros )
    #  condiciones << " and fecha >=  '" + parametros[:fecha_inicio] + "'" unless ( parametros[:fecha_inicio].nil? or parametros[:fecha_inicio].empty?)
    #  condiciones << " and fecha <=  '" + parametros[:fecha_fin] + "'" unless ( parametros[:fecha_fin].nil? or parametros[:fecha_fin].empty?)
    #end

    # cada campo tiene 1) su rotulo, 2) su atributo (anidado con puntos si hay subatributos). Igual que en el helper 
    campos_gastos = [	["Id", "id"], ["Libro Nombre", "libro.nombre"], ["Fecha", "fecha"] , ["Impuestos", "impuestos"], ["Importe", "importe"],
			["Moneda", "moneda.abreviatura"], ["Partida Codigo", "partida.codigo"], ["Concepto", "concepto"], ["Subpartida", "subpartida.nombre"],
			["Observaciones", "observaciones"],
			["Actividad", "actividad " + @proyecto.id.to_s, [[ "Actividad Código","actividad.codigo"], ["Actividad Importe", "importe"]]],
			["Financiador", "financiador " + @proyecto.id.to_s, [["Financiador Nombre", "agente.nombre"], ["Financiador Importe", "importe"]]]
			]

    linea_gasto = 0
    # PREPARACIÓN DE LA CABEZERA DE GASTO
    @tamano_columnas_gastos = []
    linea_gasto = preparar_cabecera(linea_gasto, hoja, campos_gastos, "gastos")
    
    @gastos = Gasto.find(:all, :include => :gasto_x_proyecto, :conditions => condiciones)

    color=true
    @gastos.each do |gasto|
      linea_gasto = inscribir_datos(linea_gasto, gasto, campos_gastos, hoja, @tamano_columnas_gastos,color)
      color = !color
    end
  end

  def exportar_gasto_agente hoja, parametros={}
    condiciones = "libro.agente_id = '" + @agente.id.to_s + "'"

    # De momento pasamos de fechas
    #if( parametros )
    #  condiciones << " and fecha >=  '" + parametros[:fecha_inicio] + "'" unless ( parametros[:fecha_inicio].nil? or parametros[:fecha_inicio].empty?)
    #  condiciones << " and fecha <=  '" + parametros[:fecha_fin] + "'" unless ( parametros[:fecha_fin].nil? or parametros[:fecha_fin].empty?)
    #end

    # cada campo tiene 1) su rotulo, 2) su atributo (anidado con puntos si hay subatributos). Igual que en el helper 
    campos_gastos = [   ["Id", "id"], ["Libro Nombre", "libro.nombre"], ["Fecha", "fecha"], ["Importe", "importe"], ["Moneda", "moneda.abreviatura"],
                        ["Impuestos", "impuestos"], ["Partida Codigo", "partida.codigo"], ["Concepto", "concepto"],
			["Num.Factura", "numero_factura"], ["Emisor", "emisor_factura"], ["DNI Emisor", "dni_emisor"],
                        ["Observaciones", "observaciones"],
			["Proyecto", "gasto_x_proyecto", [["Proyecto", "proyecto.nombre"], ["Porcentaje", "porcentaje"]]]
                        ]

    linea_gasto = 0
    # PREPARACIÓN DE LA CABEZERA DE GASTO
    @tamano_columnas_gastos = []
    linea_gasto = preparar_cabecera(linea_gasto, hoja, campos_gastos, "gastos")

    @gastos = Gasto.find(:all, :include => :libro, :conditions => condiciones)

    color=true
    @gastos.each do |gasto|
      linea_gasto = inscribir_datos(linea_gasto, gasto, campos_gastos, hoja, @tamano_columnas_gastos,color)
      color = !color
    end
  end

  def exportar_transferencia_proyecto hoja, parametros={}
    condiciones = "proyecto_id = '" + @proyecto.id.to_s + "'"

    # De momento pasamos de fechas
    #if( parametros )
    #  condiciones << " and fecha_inicio >=  '" + parametros[:fecha_inicio] + "'" unless ( parametros[:fecha_inicio].nil? or parametros[:fecha_inicio].empty?)
    #  condiciones << " and fecha_fin <=  '" + parametros[:fecha_fin] + "'" unless ( parametros[:fecha_fin].nil? or parametros[:fecha_fin].empty?)
    #end

    # Cada campo tiene 1) su rotulo, 2) su atributo (anidado con puntos si hay subatributos. Igual que en el helper
    campos_transferencia = [	["Id", "id"], ["Fecha", "fecha"], ["Importe", "importe"], ["Moneda", "moneda.abreviatura"],
				["Tasa Cambio", "tasa_cambio"], ["IBAN","iban"],["Numero de Cheque","numero_cheque"],
				["Cuenta", "libro.nombre"], ["Tipo", "tipo"], ["Entrante/Saliente", "entrante_saliente"], ["Segunda Cuenta", "libro_receptor_emisor.nombre"],
				["Financiador", "agente.nombre"], ["Observaciones", "observaciones"]
                                ]
    linea_transferencia = 0
    # PREPARACIÓN DE LA CABEZERA DE PRESUPUESTO
    @tamano_columnas_transferencias = []
    linea_transferencia = preparar_cabecera(linea_transferencia, hoja, campos_transferencia, "transferencias")

    color = true
    @transferencias = Transferencia.find(:all, :conditions => condiciones).each do |transferencia|
      linea_transferencia = inscribir_datos(linea_transferencia, transferencia, campos_transferencia, hoja, @tamano_columnas_transferencias, color)
      color = !color
    end
  end

  def exportar_matriz libro, parametros={}
    campos_objetivos = [	["Código", "codigo"], ["Descripcion", "descripcion"],
				["Hipotesis", "hipotesis", [["Hipotesis Descripción","descripcion"]]],
				["IOV", "indicador", [["IOV Código", "codigo"], ["IOV Descripcion", "descripcion"]]],
				["FV", "fuente_verificacion", [["FV Código","codigo"], ["FV Descripcion","descripcion"], ["FV Indicador Relacionado","indicador.codigo"] ] ]
				]

    campos_resultados = [	["Código", "objetivo_especifico.codigo"], ["Resultado Código", "codigo"], ["Resultado Descripcion", "descripcion"],
				["RH", "hipotesis", [["RH Descripción","descripcion"]]],
				["RIOV", "indicador", [["RIOV Código","codigo"], ["RIOV Descripcion", "descripcion"]]],
				["RFV", "fuente_verificacion", [["RFV Código", "codigo"], ["RFV Descripcion", "descripcion"], ["RFV Indicador Relacionado", "indicador.codigo"] ] ]
				]

    campos_actividades = [	["Resultado Código", "resultado.codigo", true], ["Actividad Código", "codigo"], ["Actividad Descripcion", "descripcion"] ]

    linea_objetivo = linea_resultado = linea_actividad = 0
    # PREPARACIÓN CABECERA DE OBJETIVOS
    @tamano_columnas_objetivos = []
    page_objetivos = libro.create_worksheet :name => "objetivos especificos"
    linea_objetivo = preparar_cabecera(linea_objetivo, page_objetivos, campos_objetivos, "objetivos")

    # PREPARACIÓN CABECERA DE RESULTADOS
    @tamano_columnas_resultados = []
    page_resultados = libro.create_worksheet :name => "resultados"
    linea_resultado = preparar_cabecera(linea_resultado, page_resultados, campos_resultados, "resultados")

    # PREPARACIÓN CABECERA DE ACTIVIDADES
    @tamano_columnas_actividades = []
    page_actividades = libro.create_worksheet :name => "actividades"
    linea_actividad = preparar_cabecera(linea_actividad, page_actividades, campos_actividades, "actividades")

    objetivos = []
    color = true 
    ObjetivoEspecifico.find_all_by_proyecto_id(@proyecto.id).each do |objetivo|
      unless objetivos.include?(objetivo.codigo)
        linea_objetivo = inscribir_datos(linea_objetivo, objetivo, campos_objetivos, page_objetivos, @tamano_columnas_objetivos,color)
        objetivos += [objetivo.codigo]
        color = !color
      end
    end

    resultados = []
    color = true 
    Resultado.find_all_by_proyecto_id(@proyecto.id).each do |resultado|
      unless resultados.include?(resultado.codigo)
        linea_resultado = inscribir_datos(linea_resultado, resultado, campos_resultados, page_resultados, @tamano_columnas_resultados,color)
        resultados += [resultado.codigo]
        color = !color
      end
    end

    color = true 
    Actividad.find_all_by_proyecto_id(@proyecto.id).each do |actividad|
      linea_actividad = inscribir_datos(linea_actividad, actividad, campos_actividades, page_actividades, @tamano_columnas_actividades,color)
        color = !color
    end
  end
  

  def preparar_cabecera(linea, page, campos, objeto)
    numero_de_columna = 0
    campos.each do |campo|
      if campo[2] and campo[2].class.to_s == "Array"
        campo[2].each do |c|
          page[linea,numero_de_columna] = c[0]
          eval("@tamano_columnas_" + objeto)[numero_de_columna] = c[0].length
          page.column(numero_de_columna).width = eval("@tamano_columnas_" + objeto)[numero_de_columna] + 1
          numero_de_columna = numero_de_columna + 1
        end
      else
        page[linea,numero_de_columna] = campo[0]
        eval("@tamano_columnas_" + objeto)[numero_de_columna] = campo[0].length
        page.column(numero_de_columna).width = eval("@tamano_columnas_" + objeto)[numero_de_columna] + 1
        numero_de_columna = numero_de_columna + 1
      end
    end
    format = Spreadsheet::Format.new :weight => :bold
    page.row(0).default_format = format
    return linea + 1
  end

  def inscribir_datos(linea_objeto, objeto, campos, page, tamano_columnas, color=false)
    linea_max = 0 # linea de altura max alcazada por un array en una linea de objeto
    numero_de_columna = 0
    linea = linea_objeto # posición inicial
    campos.each do |campo|
      if campo[2].class.to_s == "Array" # ej. campo[2] => [["IOV Código", "codigo"], ["IOV Descripcion", "descripcion"]]
        campo[2].each do |c| # ej. c => ["IOV Código", "codigo"]
          # se inicia una columna a cada miembro del array
          linea_objeto = linea # volvemos a la posición inicial a iniciar una columna
          objeto_atribu_1 = ( campo[1] =~ /(\S+)\s(\S+)/ ? objeto.send($1,$2) : objeto.send(campo[1]) )
          objeto_atribu_1.each do |obj| # obtenemos un array por ej. resultado.indicador
            # por cada miembro de array suplementario linea_objeto tiene que ser incrementada
            contenido = obj
            c[1].split('.').each do |metodo|
              contenido = ( metodo =~ /(\S+)\s(\S+)/ ? contenido.send($1,$2) : contenido.send(metodo) ) if contenido
            end 
            page[linea_objeto, numero_de_columna] = contenido
            if( contenido and contenido.to_s.length > tamano_columnas[numero_de_columna] and contenido.to_s.length <= TAMANO_MAXIMUM_COLUMNA )
              tamano_columnas[numero_de_columna] = contenido.to_s.length
              page.column(numero_de_columna).width = tamano_columnas[numero_de_columna]
            end
            page.row(linea_objeto).default_format = @format_color if color
            linea_max = ((linea_objeto - linea) > linea_max) ? linea_objeto - linea : linea_max
            linea_objeto += 1
          end
          linea_objeto = linea + 1
          numero_de_columna += 1
        end
      elsif campo[2].class.to_s == "String"
        page.row(linea_objeto).default_format = @format_color if color
        valor_campo_actual = objeto
        campo[1].split('.').each do |metodo|
          valor_campo_actual = ( metodo =~ /(\S+)\s(\S+)/ ? valor_campo_actual.send($1,$2) : valor_campo_actual.send(metodo) ) if valor_campo_actual 
        end
        if campo[3]
          @no_repetido["cadena-" + campo[1] + campo[2]] = "" unless @no_repetido["cadena-" + campo[1] + campo[2]]
          page[linea_objeto, numero_de_columna] = valor_campo_actual == @no_repetido["cadena-" + campo[1] + campo[2]] ? "" : valor_campo_actual
          @no_repetido["cadena-" + campo[1] + campo[2]] = valor_campo_actual
        else
          page[linea_objeto, numero_de_columna] = valor_campo_actual
        end
        if( page[linea_objeto, numero_de_columna] and page[linea_objeto, numero_de_columna].to_s.length > tamano_columnas[numero_de_columna] and page[linea_objeto, numero_de_columna].to_s.length <= TAMANO_MAXIMUM_COLUMNA )
          tamano_columnas[numero_de_columna] = page[linea_objeto, numero_de_columna].to_s.length
          page.column(numero_de_columna).width = tamano_columnas[numero_de_columna]
        end
        numero_de_columna += 1
      else
        page.row(linea_objeto).default_format = @format_color if color
        contenido_tmp = objeto
        campo[1].split('.').each do |metodo|
          contenido_tmp = ( metodo =~ /(\S+)\s(\S+)/ ? contenido_tmp.send($1,$2) : contenido_tmp.send(metodo) ) if contenido_tmp
        end
        page[linea_objeto, numero_de_columna] = contenido_tmp
        if( page[linea_objeto, numero_de_columna] and page[linea_objeto, numero_de_columna].to_s.length > tamano_columnas[numero_de_columna] and page[linea_objeto, numero_de_columna].to_s.length <= TAMANO_MAXIMUM_COLUMNA )
          tamano_columnas[numero_de_columna] = page[linea_objeto, numero_de_columna].to_s.length
          page.column(numero_de_columna).width = tamano_columnas[numero_de_columna]
        end
        numero_de_columna += 1
      end
    end
    return linea += linea_max + 1
  end

end
