#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de Gastos. Este controlador es utilizado desde las secciones:
# * Sección agentes: se utiliza para la gestión de los gastos relacionados con agente.
#
class GastoAgentesController < ApplicationController

  # --
  # METODOS DE ENTRADA A PRESUPUESTO. Seleccion de ETAPA
  # ++

  # en agentes: se redirecciona por defecto a listado_etapas
  def index
    redirect_to :action => :filtrado_ordenado_iniciales
  end


  # --
  # METODOS DE GESTION DE Gasto: Listados de gasto, modificar_crear, y eliminar
  # ++

  def filtrado_ordenado_iniciales
    if @agente.etapa.empty? 
      flash[:error] = _("Tiene que definir por lo menos una etapa")
      redirect_to :controller => :datos_agente, :action => :etapas
    else
      session[:gasto_agentes_asc_desc] = "ASC" 
      session[:gasto_agentes_cadena_orden] = "fecha"
      session[:gasto_agentes_filtro_etapa] = @agente.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.id
      session[:gasto_agentes_filtro_libro] = "todos"
      redirect_to :action => :listado
    end
  end

  # en agentes: establece los parametros de ordenación
  def ordenado
    session[:gasto_agentes_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:gasto_agentes_cadena_orden] = session[:gasto_agentes_orden] = params[:orden] ? params[:orden] : "fecha"
    session[:gasto_agentes_cadena_orden] = "partida.codigo" if session[:gasto_agentes_orden] == "partida.codigo_nombre"
    redirect_to :action => "listado"
  end

  def filtrado
    session[:gasto_agentes_filtro_etapa] = params[:filtro][:etapa]
    session[:gasto_agentes_filtro_moneda] = params[:filtro][:moneda]
    session[:gasto_agentes_filtro_libro] = params[:filtro][:libro]
    redirect_to :action => :listado
  end

 def elementos_filtrado
    filtro_etapa = @agente.etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    filtro_libro = [["Todos", "todos"]]  + @agente.libro.collect{ |e| [e.nombre, e.id] }
    @opciones_filtrado = [{:rotulo =>  _("Selecciona etapa: "), :nombre => "etapa", :opciones => filtro_etapa},
                         {:rotulo =>  _("Selecciona cuenta: "), :nombre => "libro", :opciones => filtro_libro}]

    @estado_filtrado = [ @etapa.nombre + " (" + @etapa.fecha_inicio.to_s + "/" + @etapa.fecha_fin.to_s + ")", 
                       (session[:gasto_agentes_filtro_libro] == "todos" ? "Cualquier cuenta" : Libro.find(session[:gasto_agentes_filtro_libro]).nombre)] 
  end

  # en agentes: lista en función de la etapa y del agente
  def listado
    # en agentes: en caso de que la seccion sea agente hacemos una ultima transformacion a los elementos del listado para multipicarlos por el porcentaje.
    # Si el parametro select del find funcionase mejor no haria falta hacer a posteriori esto.
  
    @etapa = Etapa.find( session[:gasto_agentes_filtro_etapa])
    condiciones = {"libro.agente_id" => @agente.id, "gasto.fecha" => @etapa.fecha_inicio..@etapa.fecha_fin }
    condiciones["gasto.libro_id"] = session[:gasto_agentes_filtro_libro] unless session[:gasto_agentes_filtro_libro] == "todos"
    @gastos = Gasto.paginate :all, :order => session[:gasto_agentes_cadena_orden] + " " + session[:gasto_agentes_asc_desc], 
                             :include => [:partida, :gasto_x_proyecto, :libro],
                             :conditions => condiciones, :page => params[:page], :per_page => session[:gasto_numero]
    elementos_filtrado
  end

  # en agentes: prepara el formulario de edición o creación
  def editar_nuevo
    @gasto = @objeto = params[:id] ?  Gasto.find(params[:id]) : Gasto.new
    @etapa = Etapa.find( session[:gasto_agentes_filtro_etapa] )
    @partidas = Partida.find(:all).collect {|a| [a.codigo_nombre, a.id]}
    @libros = @agente.libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}
    @proyectos = @gasto.gasto_x_proyecto @proyecto
    render :partial => "formulario"
  end

  # en agentes: modifica o crea comprobando que las fechas de la etapa coinciden
  def modificar_crear
    etapa = Etapa.find( session[:gasto_agentes_filtro_etapa] ) 
    gasto = params[:id] ?  Gasto.find(params[:id]) : Gasto.new
    gasto.attributes = params[:gasto]
    #gasto.update_attributes params[:gasto] if gasto.comprobar_fecha_etapa etapa
    gasto.update_attributes params[:gasto]
    gasto.actualizar_gasto_x_proyecto params[:proyectos] if gasto.errors.empty?
    # Genera un pago unico con fecha del gasto si se ha seleccionado y el gasto es nuevo
    if ( gasto.errors.empty? && params[:id] ) #&& params[:pago][:total] == 1 )
      pago = Pago.new
      pago.importe = gasto.importe
      pago.fecha = gasto.fecha
      pago.gasto = gasto
      pago.save
    end
    flash[:error] = gasto
    redirect_to :action => "ordenado"
  end

  # en agentes: elimina
  def eliminar
    @gasto = Gasto.find(params[:id])
    @gasto.destroy
    flash[:error_eliminar] = @gasto
    redirect_to :action => 'ordenado'
  end

  # --
  # METODOS AJAX DEL FORMULARIO: Moneda y subpartida.
  # ++

  # en agentes:  devuelve la moneda del libro seleccionado para el fomulario
  def moneda_libro
    @moneda = params[:id] != "" ? Libro.find(params[:id]).moneda : nil
    render :inline => <<-FIN
      <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
      <%= hidden_field('gasto', 'moneda_id', :value => @moneda.id) if @moneda%>
    FIN
  end

  # en agentes: devuelve las subpartidas asociadas a la partida seleccionada para el fomulario
  def subpartida
    render :partial => "subpartida", :locals => { :proyecto_id => params[:id], :linea => params[:linea]}
  end

  # --
  # METODOS AJAX DEL FORMULARIO PARA PROYECTOS
  # ++

  # en agentes: muestra el formulario para añadir proyectos
  def proyectos
    @gasto = @objeto = Gasto.find(params[:id])
    @proyectos = @gasto.gasto_x_proyecto @proyecto
    render :partial => "proyectos"
  end

  # en agentes: guarda los proyectos asociados a gasto.
  def guardar_proyectos
    @gasto = @objeto = Gasto.find(params[:id])
    @gasto.actualizar_gasto_x_proyecto params[:proyectos]
    render :update do |page|
      page.replace 'formulario', :inline => '<%= mensaje_error(@gasto) %><br>'
      #page.visual_effect :highlight, @gasto.id.to_s, :duration => 6
    end

  end

  # en agentes: añade un proyecto al formulario
  def anadir_proyecto
    @objeto = Gasto.find(params[:id])
    render :partial => "proyecto", :locals => { :linea => params[:linea].to_i, :ultima => true}
  end

end
