#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la importacion y sincronizacion de la matriz, gastos y presupuestos de un proyecto. Este controlador es utilizado desde las seccion de proyectos. Tiene 3 grupos de metodos: 
# * Los encargados de la importacion de matriz.
# * Los encargados de la importación de presupuesto.
# * Los metodos encargados de la importación de gasto.

class ImportacionController < ApplicationController
  require 'spreadsheet'

  # La entrada redirecciona a la view seleccionar para seleccionar los criterios de importación.
  def index
    @proyectos = ProyectoXImplementador.find(:all, :conditions => { :agente_id => @agente.id }).collect { |pxi| [pxi.proyecto.nombre, pxi.proyecto_id] } if params[:seccion] == "agentes"
    @agentes = ProyectoXImplementador.find(:all, :conditions => { :proyecto_id => @proyecto.id }).collect {|pxi| [pxi.agente.nombre, pxi.agente_id] } if params[:seccion] == "proyectos"
    @etapas = @proyecto ? @proyecto.etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] } : 
		Proyecto.find(@proyectos[0][1]).etapa.collect {|e| [e.nombre + " ( " + e.fecha_inicio.to_s + "  "+ e.fecha_fin.to_s + " ) ", e.id]} 

    if params[:selector] && params[:selector][:fichero]
      begin
        book = Spreadsheet.open params[:selector][:fichero]
        carga_total = params[:selector][:total]=="1" ? true : false
        etapa = carga_total ? Etapa.find(params[:etapa][:id]) : nil
        proyecto = (carga_total && params[:proyecto]) ? Proyecto.find(params[:proyecto][:id]) : nil
        agente = (carga_total && params[:agente]) ? Agente.find(params[:agente][:id]) : nil

        case params[:selector][:tipo]
          # Transferencia tiene 1 hoja
          when "transferencia" then
            importa_transferencias_proyecto book.worksheet(0), carga_total, etapa if params[:seccion] == "proyectos"
          # Presupuesto tiene 1 hoja
          when "presupuesto" then 
            importa_presupuestos_proyecto book.worksheet(0), carga_total, etapa, agente if params[:seccion] == "proyectos"
            importa_presupuestos_agente book.worksheet(0), carga_total, etapa, proyecto if params[:seccion] == "agentes"
          # Gasto tiene 1 hoja
          when "gasto" then 
            importa_gastos_proyecto book.worksheet(0), carga_total, etapa, agente if params[:seccion] == "proyectos"
            importa_gastos_agente book.worksheet(0), carga_total, etapa, proyecto if params[:seccion] == "agentes"
          # Matriz tiene 3 hojas
          when "matriz" then 
            if params[:seccion] == "proyectos"
              if book.worksheets.length < 3
                importacion_error _("ERROR: La hoja de cálculo a importar no es de Matriz")
              else
                importa_matriz_objetivos book.worksheet 0
                importa_matriz_resultados book.worksheet 1 
                importa_matriz_actividades book.worksheet 2
              end
            else
              importacion_error _("No se puede importar la matriz desde la seccion Agentes")
            end
        end
        @import_error ||= ""
      rescue Ole::Storage::FormatError
        importacion_error _("ERROR: El documento no es una hoja de cálculo válida")
      rescue
        importacion_error _("Error inesperado: Contacte con el administrador del sistema.")
      end
    end
  end

  #--
  # METODOS DE IMPORTACIÓN TRANSFERENCIAS 
  #++

  # Importa una hoja de gasto para un agente
  def importa_transferencias_proyecto hoja, carga_total=false, etapa=nil
    # El orden de las columnas es:
    # simples ->        id | libro.nombre | fecha | impuestos | importe | moneda | partida.codigo | concepto | subpartida | observaciones
    columnas = [ "id", "fecha", "importe", "moneda", "tasa_cambio", "iban", "numero_cheque", "libro.nombre", "tipo", "entrante_saliente", "libro_receptor_emisor.nombre", "financiador", "observaciones"]
    indice = Hash[*columnas.enum_with_index.to_a.flatten]
    transferencia = nil

    # Metemos todo en una transaccion para hacer rollback si es carga total y ha habido algun problema
    Transferencia.transaction do
      # Si es una carga total, deberiamos eliminar todos los elementos relacionados
      Transferencia.find(:all, 
                :conditions => {:fecha => etapa.fecha_inicio..etapa.fecha_fin, "proyecto_id" => @proyecto.id}).each { |t| t.destroy } if carga_total
      # Itera cada fila 
      hoja.each_with_index do |fila,numero_fila|
        # Si existe alguno de los datos basicos, es una fila nueva => obtenemos la transferencia 
        transferencia = actualiza_transferencia_proyecto(indice, fila, etapa) if numero_fila > 0
      end
      # Si hubo algun error y seleccionamos carga total, deshacemos los cambios
      raise(ActiveRecord::Rollback, "Hacemos un rollback") if carga_total && @import_error != ""
    end
  end

  def actualiza_transferencia_proyecto indice, fila, etapa
    transferencia = nil
    libro = @proyecto.libro.find_by_nombre fila[indice['libro.nombre']]
    libro_receptor_emisor = @proyecto.libro.find_by_nombre fila[indice['libro_receptor_emisor.nombre']] if fila[indice['libro_receptor_emisor.nombre']]
    financiador = @proyecto.financiador.find_by_nombre fila[indice['financiador']] if fila[indice['financiador']]

    # La moneda es la del libro, pasamos de lo que haya en el campo
    fecha_transf = etapa ? (fecha fila[indice['fecha']] if fila[indice['fecha']] >= etapa.fecha_inicio && fila[indice['fecha']] <= etapa.fecha_fin) : fecha(fila[indice['fecha']])

    if libro && fecha_transf && (financiador || !fila[indice['financiador']]) && ( libro_receptor_emisor || !fila[indice['libro_receptor_emisor.nombre']] )
      transferencia = fila[indice['id']] ? @proyecto.transferencia.find_by_id(fila[indice['id']]) : Transferencia.new
      # Si acabamos de crear el gasto o existia pero solo esta asignado a este proyecto...
      if transferencia
        transferencia.proyecto_id = @proyecto.id unless fila[indice['id']]
        transferencia.libro_id = libro.id
        transferencia.libro_receptor_emisor_id = libro_receptor_emisor.id if libro_receptor_emisor
        transferencia.fecha = fecha_transf
        transferencia.importe = numero fila[indice['importe']]
        transferencia.moneda_id = libro.moneda.id
        transferencia.tasa_cambio = numero fila[indice['tasa_cambio']]
        transferencia.numero_cheque = fila[indice['numero_cheque']]
        transferencia.iban = fila[indice['iban']]
        transferencia.tipo = fila[indice['tipo']].to_s.downcase
        transferencia.entrante_saliente = fila[indice['entrante_saliente']].to_s.downcase
        transferencia.agente_id = financiador.id if financiador
        transferencia.observaciones =  fila[indice['observaciones']]
        transferencia.save
        importacion_error transferencia, _("Transferencia") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "'" : "nueva")
      # Si no se ha encontrado o no se ha podido crear la transferencia 
      else
        importacion_error "<br>" + _("Se produjeron errores procesando Transferencia ") + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "':" : _("nueva:"))
        importacion_error _("La transferencia no existe en este proyecto.<br>Si es una transferencia nueva, elimine el ID para crearla.") unless transferencia
      end
    else
      importacion_error "<br>" + _("Se produjeron errores procesando Transferencia ") + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "':" : _("nueva:"))
      importacion_error _("Libro") + " '" + (fila[indice['libro.nombre']].to_s||"") + "' " + _("no encontrado o no válido.") unless libro
      importacion_error _("Libro") + " " + _("receptor/emisor") + " '" + fila[indice['libro_receptor_emisor.nombre']].to_s + "' " + _("no encontrado o no válido.") unless ( libro_receptor_emisor || !fila[indice['libro_receptor_emisor.nombre']] )
      importacion_error _("Fecha") + " '" + (fila[indice['fecha']].to_s||"") + "' " + _("no válida para la etapa.") unless fecha_transf
      importacion_error _("Financiador") + " '" + fila[indice['financiador']].to_s + "' " + _("no encontrado o no válido.") unless ( financiador || !fila[indice['financiador']] )
    end

    return transferencia 
  end

  #--
  # METODOS DE IMPORTACIÓN GASTOS
  #++

  # Importa una hoja de gasto para un agente
  def importa_gastos_agente hoja, carga_total=false, etapa=nil, proyecto=nil
    # El orden de las columnas es:
    # simples ->        id | libro.nombre | fecha | impuestos | importe | moneda | partida.codigo | concepto | subpartida | observaciones
    # multiples ->      actividad.codigo | actividad.importe | actividad.unidades
    # multiples ->      financiador.nombre | financiador.importe
    columnas = [ 'id','libro.nombre','fecha','importe','moneda','impuestos','partida.codigo','concepto','numero_factura','emisor_factura','dni_emisor','observaciones',
		'proyecto.nombre','proyecto.porcentaje' ]
    indice = Hash[*columnas.enum_with_index.to_a.flatten]
    gasto = nil

    # Metemos todo en una transaccion para hacer rollback si es carga total y ha habido algun problema
    Gasto.transaction do
      # Si es una carga total, deberiamos eliminar todos los elementos relacionados
      Gasto.find(:all, :include => [:gasto_x_proyecto], 
		:conditions => {:fecha => etapa.fecha_inicio..etapa.fecha_fin, "gasto_x_proyecto.proyecto_id" => proyecto.id}).each { |g| g.destroy } if carga_total
      # Itera cada fila 
      hoja.each_with_index do |fila,numero_fila|
        # Si existe alguno de los datos basicos, es una fila nueva => obtenemos el gasto 
        gasto = actualiza_gasto_agente(indice, fila, etapa) if (fila[indice['libro.nombre']] || fila[indice['moneda']] || fila[indice['partida.codigo']]) && numero_fila > 0
        if ( gasto )
          actualiza_gasto_agente_proyecto gasto, indice, fila, proyecto if fila[indice['proyecto.nombre']]
        end
      end
      # Si hubo algun error y seleccionamos carga total, deshacemos los cambios
      raise(ActiveRecord::Rollback, "Hacemos un rollback") if carga_total && @import_error != ""
    end
  end

  def actualiza_gasto_agente indice, fila, etapa
    gasto = nil
    libro = @agente.libro.find_by_nombre fila[indice['libro.nombre']]
    # La moneda es la del libro, pasamos de lo que haya en el campo
    moneda = libro.moneda if libro
    partida = Partida.find_by_codigo fila[indice['partida.codigo']]
    fecha_gasto = fecha(fila[indice['fecha']]) if etapa.nil? || (fecha(fila[indice['fecha']]) >= etapa.fecha_inicio && fecha(fila[indice['fecha']]) <= etapa.fecha_fin)
    # En el caso de que sea una importacion total, comprobamos que el gasto este asignado a un proyecto
    comprobar_proyecto = etapa ? fila[indice['proyecto.nombre']] : "true" 

    if libro && moneda && partida && fecha_gasto && comprobar_proyecto
      # el gasto lo encontramos por su relacion con el libro
      gasto = fila[indice['id']] ? libro.gasto.find_by_id(fila[indice['id']]) : Gasto.new
      # Si acabamos de crear el gasto o ya existia
      if gasto
        gasto.libro_id = libro.id
        gasto.fecha = fecha_gasto 
        # Ojo!!!, para el importe hay que tener en cuenta el porcentaje que marca gasto_x_proyecto
        gasto.importe = numero fila[indice['importe']]
        gasto.moneda_id = moneda.id
        gasto.impuestos = numero fila[indice['impuestos']]
        gasto.partida_id = partida.id
        gasto.concepto = fila[indice['concepto']].to_s
        gasto.observaciones = fila[indice['observaciones']].to_s
        gasto.numero_factura = fila[indice['numero_factura']].to_s
        gasto.emisor_factura = fila[indice['emisor_factura']].to_s
        gasto.dni_emisor = fila[indice['dni_emisor']].to_s
        gasto.save
        importacion_error gasto, _("Gasto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "'" : "nuevo")
      else
        importacion_error "<br>" + _("Se produjeron errores procesando Gasto ") + _("con ID") + " '" + fila[indice['id']].to_s + "':"
        importacion_error _("No se ha encontrado el gasto con ese ID en el libro seleccionado.") 
      end
    else
      importacion_error "<br>" + _("Se produjeron errores procesando Gasto ") + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "':" : _("nuevo:"))
      importacion_error _("Libro") + " '" + (fila[indice['libro.nombre']].to_s||"") + "' " + _("no encontrado o no válido.") unless libro
      #importacion_error _("Moneda") + " '" + (fila[indice['moneda']]||"") + "' " + _("no encontrada o no válida.") unless moneda
      importacion_error _("Partida") + " '" + (fila[indice['partida.codigo']].to_s||"") + "' " + _("no encontrada o no válida.") unless partida
      importacion_error _("Fecha") + " '" + (fila[indice['fecha']].to_s||"") + "' " + _("no válida para la etapa.") unless fecha_gasto
      importacion_error _("Proyecto no definido") unless comprobar_proyecto
    end

    return gasto
  end

  def actualiza_gasto_agente_proyecto gasto, indice, fila, total_proyecto
    #proyecto=gasto.libro.proyecto.find_by_nombre fila[indice['proyecto.nombre']]
    proyecto = gasto.libro.proyecto.find_by_nombre fila[indice['proyecto.nombre']] if total_proyecto.nil? || fila[indice['proyecto.nombre']] == total_proyecto.nombre
    # Se asegura que el agente sea implementador del proyecto
    proyecto = nil if proyecto && proyecto.implementador.find(@agente.id).nil?
    if proyecto && gasto
      gxp = gasto.gasto_x_proyecto.find_by_proyecto_id proyecto.id 
      gxp = gasto.gasto_x_proyecto.new(:proyecto_id => proyecto.id) if gxp.nil?
      gxp.porcentaje = numero fila[indice['proyecto.porcentaje']]
      gxp.save
      importacion_error gxp, _("Proyecto") + " '" + proyecto.nombre + "' " + _("en el Gasto con ID") + " '" + gasto.id.to_s
    else
      importacion_error "<br>" + _("Se produjeron errores asignando el Gasto con ID") + " '" + gasto.id.to_s + "' " + _("al proyecto") + " '" + fila[indice['proyecto.nombre']] + "':"
      importacion_error _("El libro") + " '" + gasto.libro.nombre + "' " + _("no es válido para el proyecto, el agente no es implementador del proyecto o el proyecto no existe") if total_proyecto.nil? || fila[indice['proyecto.nombre']] == total_proyecto.nombre
      importacion_error _("El proyecto no coincide con el seleccionado para importacion") if total_proyecto && fila[indice['proyecto.nombre']] != total_proyecto.nombre
    end
  end

  # Importa una hoja de gasto para un proyecto
  def importa_gastos_proyecto hoja, carga_total=false, etapa=nil, agente=nil
    # El orden de las columnas es:
    # simples ->        id | libro.nombre | fecha | impuestos | importe | moneda | partida.codigo | concepto | subpartida | observaciones
    # multiples ->      actividad.codigo | actividad.importe | actividad.unidades
    # multiples ->      financiador.nombre | financiador.importe
    columnas = [ 'id','libro.nombre','fecha','impuestos','importe','moneda','partida.codigo','concepto','subpartida','observaciones','actividad.codigo','actividad.importe','financiador.nombre','financiador.importe' ]
    indice = Hash[*columnas.enum_with_index.to_a.flatten]
    gasto = nil

    # Metemos todo en una transaccion para hacer rollback si es carga total y ha habido algun problema
    Gasto.transaction do 
      # Si es una carga total, borramos todos los gastos del proyecto
      @proyecto.gasto.find(:all, :include => "libro", :conditions => {'libro.agente_id' => agente.id, :fecha => etapa.fecha_inicio..etapa.fecha_fin}).each { |eliminar_gasto| eliminar_gasto.destroy } if carga_total
      # Itera cada fila 
      hoja.each_with_index do |fila,numero_fila|
        # Si existe alguno de los datos basicos, es una fila nueva => obtenemos el gasto 
        gasto = actualiza_gasto_proyecto indice, fila, etapa, agente if (fila[indice['libro.nombre']] || fila[indice['moneda']] || fila[indice['partida.codigo']]) && numero_fila > 0
        if ( gasto )
          actualiza_gasto_actividad gasto, indice, fila if fila[indice['actividad.codigo']]
          actualiza_gasto_financiador gasto, indice, fila if fila[indice['financiador.nombre']]
        end
      end
      # Si hubo algun error y seleccionamos carga total, deshacemos los cambios
      raise(ActiveRecord::Rollback, "Hacemos un rollback") if carga_total && @import_error != ""
    end
  end

  def actualiza_gasto_proyecto indice, fila, etapa, agente_total
    gasto = nil
    libro = Libro.find_by_nombre fila[indice['libro.nombre']], :include => "proyecto", :conditions => { "proyecto.id" => @proyecto.id }
    # Se asegura de que el libro pertenezca al implantador seleccionado (cuando hay carga total)
    libro = nil if libro && agente_total && libro.agente_id != agente_total.id

    # La moneda es la del libro, pasamos de lo que haya en el campo
    moneda = libro.moneda if libro
    partida = Partida.find_by_codigo fila[indice['partida.codigo']] 
    subpartida = Subpartida.find_by_nombre fila[indice['subpartida']]
    fecha_gasto = etapa ? (fecha fila[indice['fecha']] if fila[indice['fecha']] >= etapa.fecha_inicio && fila[indice['fecha']] <= etapa.fecha_fin) : fecha(fila[indice['fecha']])

    if libro && moneda && partida && fecha_gasto
      gasto = fila[indice['id']] ? @proyecto.gasto.find_by_id(fila[indice['id']]) : Gasto.new
      # Si acabamos de crear el gasto o existia pero solo esta asignado a este proyecto...
      if gasto && (gasto.id.nil? || GastoXProyecto.find(:all, :conditions => { :gasto_id => gasto.id }).length == 1)
        gasto.gasto_x_proyecto << GastoXProyecto.new(:proyecto_id => @proyecto.id, :porcentaje => 1) unless fila[indice['id']] 
        gasto.libro_id = libro.id
        gasto.fecha = fecha_gasto 
        gasto.impuestos = numero fila[indice['impuestos']]
        # Ojo!!!, para el importe hay que tener en cuenta el porcentaje que marca gasto_x_proyecto
        gasto.importe = numero fila[indice['importe']]
        gasto.moneda_id = moneda.id 
        gasto.partida_id = partida.id
        gasto.concepto = fila[indice['concepto']].to_s
        gasto.subpartida_id = subpartida.id if subpartida
        gasto.observaciones = fila[indice['observaciones']].to_s
        gasto.save
        importacion_error gasto, _("Gasto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "'" : "nuevo")
      # Si no se ha encontrado el gasto o esta compartido por varios proyectos...
      else
        importacion_error "<br>" + _("Se produjeron errores procesando Gasto ") + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "':" : _("nuevo:"))
        importacion_error _("El gasto no existe en este proyecto.<br>Si es un gasto nuevo, elimine el ID para crearlo.") unless gasto
        importacion_error _("El gasto está compartido por varios proyectos. Solo puede modificarse desde la sección Agentes.") if gasto
      end
    else
      importacion_error "<br>" + _("Se produjeron errores procesando Gasto ") + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "':" : _("nuevo:"))
      importacion_error _("Libro") + " '" + (fila[indice['libro.nombre']].to_s||"") + "' " + _("no encontrado o no válido.") unless libro
      #importacion_error _("Moneda '") + (fila[indice['moneda']]||"") + _("' no encontrada o no válida.") unless moneda
      importacion_error _("Partida") + " '" + (fila[indice['partida.codigo']].to_s||"") + "' " + _("no encontrada o no válida.") unless partida
      importacion_error _("Fecha") + " '" + (fila[indice['fecha']].to_s||"") + "' " + _("no válida para la etapa.") unless fecha_gasto
    end
    if fila[indice['subpartida']]
      importacion_error _("(error no crítico) Subpartida '") + fila[indice['subpartida']].to_s + "' " +  _("no encontrada o no válida para gasto ") + (fila[indice['id']] ? _("con id") + " '" + fila[indice['id']].to_s + "'." : "nuevo") unless subpartida
    end

    return gasto
  end

  def actualiza_gasto_actividad gasto, indice, fila
    actividad = Actividad.find_by_codigo fila[indice['actividad.codigo']], :conditions => { :proyecto_id => @proyecto.id }
    if actividad && gasto
      if ! (gxa = GastoXActividad.find :first, :conditions => { :gasto_id => gasto.id, :actividad_id => actividad.id, :proyecto_id => @proyecto.id } )
        gxa = GastoXActividad.new :gasto_id => gasto.id, :actividad_id => actividad.id, :proyecto_id => @proyecto.id
      end
      gxa.importe = numero fila[indice['actividad.importe']]
      #gxa.numero_unidades = numero fila[indice['actividad.numero_unidades']]
      gxa.save
      importacion_error gxa, "Actividad '" + actividad.codigo + "' en el Gasto con ID '" + gasto.id.to_s
    elsif fila[indice['actividad.codigo']]
      importacion_error "<br>" + _("Actividad ") + fila[indice['actividad.codigo']].to_s + _("' no encontrada o no válida.")
    end
  end

  def actualiza_gasto_financiador gasto, indice, fila
    agente = @proyecto.financiador.find_by_nombre fila[indice['financiador.nombre']]
    if agente && gasto
      if ! ( gxf = GastoXAgente.find :first, :conditions => { :agente_id => agente.id, :gasto_id => gasto.id, :proyecto_id => @proyecto.id } )
        gxf = GastoXAgente.new :gasto_id => gasto.id, :agente_id => agente.id, :proyecto_id => @proyecto.id
      end
      gxf.importe = numero fila[indice['financiador.importe']]
      gxf.save
      importacion_error gxf, "Financiador '" + agente.nombre + "' en el Gasto con ID '" + gasto.id.to_s
    elsif fila[indice['financiador.nombre']]
      importacion_error "<br>" + _("Financiador ") + fila[indice['financiador.nombre']].to_s + _(" no encontrado o no válido.")
    end
  end

  #--
  # METODOS DE IMPORTACIÓN PRESUPUESTOS
  #++

 # Importa una hoja de presupuesto para un agente
  def importa_presupuestos_agente hoja, carga_total=false, etapa=nil, proyecto=nil
    # El orden de las columnas es:
    # simples ->        id | proyecto.nombre | etapa | unidad | numero_unidades | coste_unitario | importe | moneda.abreviatura | partida.codigo | subpartida | concepto | observ.
    columnas = [ 'id','proyecto.nombre','etapa.nombre','unidad','numero_unidades','coste_unitario','importe','moneda.abreviatura','partida.codigo','subpartida','concepto','observaciones' ]
    indice=Hash[*columnas.enum_with_index.to_a.flatten]

    presupuesto = nil

    # Metemos todo en una transaccion para hacer rollback si es carga total y ha habido algun problema
    Presupuesto.transaction do
      # Si es una carga total borramos todos los presupuestos del proyecto y el agente
      Presupuesto.find( :all, :conditions => { :etapa_id => etapa.id ,
			:agente_id => @agente.id, :proyecto_id => proyecto.id }  ).each { |p| p.destroy } if carga_total
      # Itera cada fila 
      hoja.each_with_index do |fila,numero_fila|
        # Si existe alguno de los datos basicos => obtenemos el presupuesto 
        presupuesto = actualiza_presupuesto_agente indice, fila, etapa, proyecto if (fila[indice['proyecto.nombre']] || fila[indice['concepto']] || fila[indice['partida.codigo']] || fila[indice['importe']] || fila[indice['numero_unidades']] || fila[indice['moneda.abreviatura']]) && numero_fila > 0
      end
      raise(ActiveRecord::Rollback, "Hacemos un rollback") if carga_total && @import_error != ""
    end
  end

  def actualiza_presupuesto_agente indice, fila, etapa, total_proyecto
    presupuesto = nil
    proyecto = Proyecto.find_by_nombre fila[indice['proyecto.nombre']] if total_proyecto.nil? || fila[indice['proyecto.nombre']] == total_proyecto.nombre
    # Se asegura que el agente sea implementador del proyecto
    proyecto = nil if proyecto && proyecto.implementador.find(@agente.id).nil?
    concepto = fila[indice['concepto']].to_s
    importe = numero fila[indice['importe']]
    mon = Moneda.find_by_abreviatura fila[indice['moneda.abreviatura']]
    moneda = mon if mon && !@agente.libro.find_by_moneda_id(mon.id).nil?
    numero_unidades = numero fila[indice['numero_unidades']]
    partida = Partida.find_by_codigo fila[indice['partida.codigo']]

    if proyecto && moneda && partida && etapa 
      # como ya hemos filtrado proyecto no necesitamos afinar en la busqueda de presupuesto
      presupuesto = fila[indice['id']] ? proyecto.presupuesto.find_by_id(fila[indice['id']]) : Presupuesto.new( :proyecto_id => proyecto.id, :agente_id => @agente.id )
      presupuesto = nil if presupuesto && presupuesto.agente_id != @agente.id
      if !presupuesto.nil?
        presupuesto.fecha_inicio = fecha_inicio 
        presupuesto.fecha_fin = fecha_fin
        presupuesto.unidad = fila[indice['unidad']]
        presupuesto.numero_unidades = numero fila[indice['numero_unidades']]
        presupuesto.coste_unitario = fila[indice['coste_unitario']]
        presupuesto.importe = numero fila[indice['importe']]
        presupuesto.moneda = moneda
        presupuesto.partida = partida
        presupuesto.subpartida = Subpartida.find_by_nombre fila[indice['subpartida']]
        presupuesto.concepto = fila[indice['concepto']].to_s
        presupuesto.observaciones = fila[indice['observaciones']].to_s
        presupuesto.save
        importacion_error presupuesto, _("Presupuesto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "'" : "nuevo")
      # Si no se ha encontrado el presupuesto para este agente...
      else
        importacion_error "<br>" + _("Se produjeron errores procesando línea de Presupuesto") + " " + _("con ID") + " '" + fila[indice['id']].to_s + ":"
        importacion_error _("Línea de presupuesto no encontrada o no válida para el agente") 
      end
    else
      importacion_error "<br>" + _("Se produjeron errores procesando línea de Presupuesto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "':" : "nuevo:")
      importacion_error _("El agente actual no es implementador en el proyecto") + " " + fila[indice['proyecto.nombre']].to_s + " " + _("o se indicó otro en el formulario de importación.") unless proyecto
      importacion_error _("Moneda") + " '" + (fila[indice['moneda.abreviatura']].to_s||"") + "' " + _("no encontrada o no válida.") unless moneda
      importacion_error _("Partida") + " '" + (fila[indice['partida.codigo']].to_s||"") + "' " + _("no encontrada o no válida.") unless partida
      importacion_error _("Fecha de Inicio") + " '" + (fila[indice['fecha_inicio']].to_s||"") + "' " + _("no válida para la etapa.") unless fecha_inicio
      importacion_error _("Fecha de Fin") + " '" + (fila[indice['fecha_fin']].to_s||"") + "' " + _("no válida para la etapa.") unless fecha_fin
    end

    return presupuesto
  end

  # Importa una hoja de presupuesto para un proyecto
  def importa_presupuestos_proyecto hoja, carga_total=false, etapa=nil, agente=nil
    # El orden de las columnas es:
    # simples ->	id | agente.nombre | etapa.nombre | unidades | coste_unitario | importe | partida.nombre | moneda.abreviatura | concepto | subpartida | observ.
    # multiples ->	actividad.codigo | actividad.importe | actividad.unidades
    # multiples ->	financiador.nombre | financiador.importe
    columnas = [ 'id','agente.nombre','etapa.nombre','unidad','numero_unidades','coste_unitario','importe','moneda.abreviatura','partida.codigo','subpartida','concepto','observaciones','actividad.codigo','actividad.importe','actividad.unidades','financiador.nombre','financiador.importe' ]
    indice=Hash[*columnas.enum_with_index.to_a.flatten]

    presupuesto = nil
    # Metemos todo en una transaccion para hacer rollback si es carga total y ha habido algun problema
    Presupuesto.transaction do
      # Si es una carga total borramos todos los presupuestos del proyecto
      Presupuesto.destroy_all(:proyecto_id => @proyecto.id, :agente_id => agente.id, :etapa_id => etapa.id) if carga_total
      # Itera cada fila 
      hoja.each_with_index do |fila,numero_fila|
        # Si existe alguno de los datos basicos, es una fila nueva => obtenemos el presupuesto 
        presupuesto = actualiza_presupuesto_proyecto indice, fila, etapa, agente if (fila[indice['agente.nombre']] || fila[indice['concepto']] || fila[indice['partida.codigo']] || fila[indice['importe']] || fila[indice['numero_unidades']] || fila[indice['moneda.abreviatura']]) && numero_fila > 0
        if ( presupuesto )
          actualiza_presupuesto_actividad presupuesto, indice, fila if fila[indice['actividad.codigo']]
          actualiza_presupuesto_financiador presupuesto, indice, fila if fila[indice['financiador.nombre']]
        end
      end
      # Si hubo algun error y seleccionamos carga total, deshacemos los cambios
      raise(ActiveRecord::Rollback, "Hacemos un rollback") if carga_total && @import_error != ""
    end
  end

  def actualiza_presupuesto_proyecto indice, fila, etapa_total, agente_total
    presupuesto = nil
    agente = @proyecto.implementador.find_by_nombre fila[indice['agente.nombre']] 
    # Para importacion total, se asegura de importar solo si el agente es el indicado
    agente = nil if agente && agente_total && agente.id != agente_total.id
    # Para importacion total, se asegura de que la etapa sea la indicada
    etapa = Etapa.find_by_nombre fila[indice['etapa.nombre']] if etapa_total.nil? || etapa_total.nombre == fila[indice['etapa.nombre']]

    moneda = @proyecto.moneda.find_by_abreviatura fila[indice['moneda.abreviatura']]
    partida = Partida.find_by_codigo fila[indice['partida.codigo']]
    subpartida = Subpartida.find :first, :conditions => { :nombre => fila[indice['subpartida']], :proyecto_id => @proyecto.id } if fila[indice['subpartida']]
    #fecha_inicio = etapa ? (fecha fila[indice['fecha_inicio']] if fila[indice['fecha_inicio']] >= etapa.fecha_inicio) : fecha(fila[indice['fecha_inicio']])
    #fecha_fin = etapa ? (fecha fila[indice['fecha_fin']] if fila[indice['fecha_fin']] <= etapa.fecha_fin) : fecha(fila[indice['fecha_fin']])

    #if agente && moneda && partida && fecha_inicio && fecha_fin
    if agente && moneda && partida && etapa
      presupuesto = fila[indice['id']] ? @proyecto.presupuesto.find_by_id(fila[indice['id']]) : Presupuesto.new
      # Si acabamos de crear el presupuesto o esta asignado unicamente al proyecto
      if presupuesto
        presupuesto.proyecto_id = @proyecto.id if presupuesto.id.nil?
        presupuesto.agente = agente 
        presupuesto.concepto = fila[indice['concepto']].to_s 
        # Ojo!!!, para el importe hay que tener en cuenta el porcentaje que marca gasto_x_proyecto
        presupuesto.importe = numero fila[indice['importe']] 
        presupuesto.moneda = moneda
        presupuesto.unidad = fila[indice['unidad']]
        presupuesto.numero_unidades = numero fila[indice['numero_unidades']] 
        presupuesto.coste_unitario = fila[indice['coste_unitario']]
        presupuesto.partida = partida
        presupuesto.subpartida = subpartida if subpartida
        #presupuesto.fecha_inicio = fecha fila[indice['fecha_inicio']]
        #presupuesto.fecha_fin = fecha fila[indice['fecha_fin']]
        presupuesto.etapa_id = etapa.id
        presupuesto.observaciones = fila[indice['observaciones']].to_s
        presupuesto.save
        importacion_error presupuesto, _("Presupuesto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "'" : "nuevo")
        # Si hemos creado correctamente el presupuesto pero la subpartida no existia, la creamos y la asignamos
        if presupuesto.errors.empty? && subpartida.nil? && fila[indice['subpartida']]
          #puts "---------------> Vamos a crear la subpartida " + fila[indice['subpartida']]
          subpart = Subpartida.new :nombre => fila[indice['subpartida']], :proyecto_id => @proyecto.id
          subpart.save
          presupuesto.subpartida = subpartida = subpart if subpart.errors.empty?
          importacion_error subpart, _("Presupuesto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "'" : "nuevo")
          presupuesto.save if subpart.errors.empty?
          importacion_error presupuesto, _("Presupuesto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "'" : "nuevo")
        end
        
      # Si no se ha encontrado el gasto o esta compartido por varios proyectos...
      else
        importacion_error "<br>" + _("Se produjeron errores procesando línea de Presupuesto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "':" : "nuevo:")
        importacion_error _("El presupuesto no existe en este proyecto.<br>Si es un presupuesto nuevo, elimine el ID para crearlo.")
      end
    else
      importacion_error "<br>" + _("Se produjeron errores procesando línea de Presupuesto ") + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "':" : "nuevo:")
      importacion_error _("Agente") + " '" + (fila[indice['agente.nombre']].to_s||"") + "' " + _("no encontrado o no válido.") unless agente
      importacion_error _("Moneda") + " '" + (fila[indice['moneda.abreviatura']].to_s||"") + "' " + _("no encontrada o no válida.") unless moneda
      importacion_error _("Partida") + " '" + (fila[indice['partida.codigo']].to_s||"") + "' " + _("no encontrada o no válida.") unless partida
      importacion_error _("Etapa") + " '" + (fila[indice['etapa.nombre']].to_s||"") + "' " + _("no valida") unless etapa
      #importacion_error _("Fecha de Inicio") + " '" + (fila[indice['fecha_inicio']].to_s||"") + "' " + _("no válida para la etapa.") unless fecha_inicio
      #importacion_error _("Fecha de Fin") + " '" + (fila[indice['fecha_fin']].to_s||"") + "' " + _("no válida para la etapa.") unless fecha_fin

    end
    if fila[indice['subpartida']]
      importacion_error _("(error no crítico) Subpartida '") + fila[indice['subpartida']].to_s + "' " + _("en la línea de Presupuesto") + " " + (fila[indice['id']] ? _("con ID") + " '" + fila[indice['id']].to_s + "'." : "nueva.") + " " + _("no encontrada o no válida.") unless subpartida
    end

    return presupuesto 
  end

  def actualiza_presupuesto_actividad presupuesto, indice, fila
    actividad = Actividad.find_by_codigo fila[indice['actividad.codigo']], :conditions => { :proyecto_id => @proyecto.id }
    if actividad && presupuesto
      if ! (presupuesto.presupuesto_x_actividad && (pxa = presupuesto.presupuesto_x_actividad.find_by_actividad_id actividad.id))
        pxa = PresupuestoXActividad.new :presupuesto_id => presupuesto.id, :actividad_id => actividad.id
      end
      pxa.importe = numero fila[indice['actividad.importe']]
      pxa.numero_unidades = numero fila[indice['actividad.unidades']]
      pxa.save
      importacion_error pxa, _("Actividad") + " '" + actividad.codigo + "' " + _("en la línea de Presupuesto") + " " + _("con ID") + " '" + presupuesto.id.to_s
    elsif fila[indice['actividad.codigo']]
      importacion_error "<br>" + _("Actividad") + " " + fila[indice['actividad.codigo']].to_s + "' " + _("no encontrada o no válida.")
    end
  end

  def actualiza_presupuesto_financiador presupuesto, indice, fila
    agente = @proyecto.financiador.find_by_nombre fila[indice['financiador.nombre']]
    if agente && @proyecto 
      if ! (presupuesto.presupuesto_x_agente && (pxf = presupuesto.presupuesto_x_agente.find_by_agente_id agente.id) )
        pxf = PresupuestoXAgente.new :presupuesto_id => presupuesto.id, :agente_id => agente.id
      end
      pxf.importe = numero fila[indice['financiador.importe']]
      pxf.save
      importacion_error pxf, _("Financiador") + " '" + agente.nombre + "' " + _("en línea de Presupuesto con ID") + " '" + presupuesto.id.to_s + "'"
    elsif fila[indice['financiador.nombre']]
      importacion_error "<br>" + _("Financiador") + " " + fila[indice['financiador.nombre']].to_s + " " + _("no encontrado o no válido.")
    end
  end

  #--
  # METODOS DE IMPORTACIÓN MATRIZ
  #++

  # Importa una hoja de objetivos para la matriz
  def importa_matriz_objetivos hoja
    # El orden de las columnas es:
    # simples ->        codigo | descripcion 
    # multiples ->      hipotesis.descripcion 
    # multiples ->      iov.codigo | iov.descripcion
    # multiples ->	fv.codigo | fv.descripcion | fv.iov.codigo
    columnas = [ 'codigo','descripcion','hipotesis.descripcion','iov.codigo','iov.descripcion','fv.codigo','fv.descripcion', 'fv.indicador' ]
    indice=Hash[*columnas.enum_with_index.to_a.flatten]

    oe = nil
    # Itera cada fila 
    hoja.each_with_index do |fila,numero_fila|
      # Si existe alguno de los datos basicos, es una fila nueva => obtenemos el oe 
      oe = actualiza_oe indice, fila if (fila[indice['codigo']] || fila[indice['descripcion']] ) && numero_fila > 0
      if ( oe )
        actualiza_hipotesis oe, indice, fila if fila[indice['hipotesis.descripcion']]
        actualiza_indicador oe, indice, fila if fila[indice['iov.codigo']] || fila[indice['iov.descripcion']]
        actualiza_fuente oe, indice, fila if fila[indice['fv.codigo']] || fila[indice['fv.descripcion']] || fila[indice['fv.indicador']]
      end
    end
  end

  # Actualiza un objetivo especifico
  def actualiza_oe indice, fila
    oe = nil
    if fila[indice['codigo']]
      oe = @proyecto.objetivo_especifico.find_by_codigo(fila[indice['codigo']]) || ObjetivoEspecifico.new
      oe.codigo = fila[indice['codigo']].to_s if oe.id.nil?
      oe.proyecto_id = @proyecto.id if oe.id.nil?
      oe.descripcion = fila[indice['descripcion']].to_s if !fila[indice['descripcion']].nil? || oe.id.nil?
      oe.save
      importacion_error oe, _("Objetivo Especifico") + " " + _("con código") + " '" + fila[indice['codigo']].to_s + "'"
    else
      importacion_error "<br>" + _("Se produjeron errores procesando Objetivo Especifico")
      importacion_error _("Objetivo Especifico debe tener un código")
    end
    return oe
  end

  # Importa una hoja de resultados para la matriz
  def importa_matriz_resultados hoja
    # El orden de las columnas es:
    # simples ->        oe.codigo | codigo | descripcion 
    # multiples ->      hipotesis.descripcion 
    # multiples ->      iov.codigo | iov.descripcion
    # multiples ->      fv.codigo | fv.descripcion | fv.iov.codigo
    columnas = [ 'oe.codigo', 'codigo','descripcion','hipotesis.descripcion','iov.codigo','iov.descripcion','fv.codigo','fv.descripcion', 'fv.indicador' ]
    indice=Hash[*columnas.enum_with_index.to_a.flatten]

    resultado = nil
    # Itera cada fila 
    hoja.each_with_index do |fila,numero_fila|
      # Si existe alguno de los datos basicos, es una fila nueva => obtenemos el oe 
      resultado = actualiza_resultado indice, fila if (fila[indice['oe.codigo']]) && numero_fila > 0
      if ( resultado )
        actualiza_hipotesis resultado, indice, fila if fila[indice['hipotesis.descripcion']]
        actualiza_indicador resultado, indice, fila if fila[indice['iov.codigo']] || fila[indice['iov.descripcion']]
        actualiza_fuente resultado, indice, fila if fila[indice['fv.codigo']] || fila[indice['fv.descripcion']] || fila[indice['fv.indicador']]
      end
    end
  end

  # Actualiza un resultado
  def actualiza_resultado indice, fila
    resultado=nil
    oe = @proyecto.objetivo_especifico.find_by_codigo(fila[indice['oe.codigo']])
    if oe && fila[indice['codigo']]
      resultado = oe.resultado.find_by_codigo(fila[indice['codigo']]) || Resultado.new
      resultado.objetivo_especifico_id = oe.id if resultado.id.nil?
      resultado.codigo = fila[indice['codigo']].to_s if resultado.id.nil?
      resultado.descripcion = fila[indice['descripcion']].to_s
      resultado.proyecto_id = @proyecto.id
      resultado.save
      importacion_error resultado, _("Resultado") + " '" + fila[indice['codigo']].to_s + "' " + _("del Objetivo Especifico") + " '" + fila[indice['oe.codigo']].to_s + "'"
    else
      importacion_error "<br>" + _("Se produjeron errores procesando Resultado") + " " + (!fila[indice['codigo']].nil? ? "<br>" + _("El Resultado debe tener un código") :  _("con código") + " '" + fila[indice['codigo']].to_s + "'")
      importacion_error _("Objetivo Especifico") + " '" + fila[indice['oe.codigo']].to_s + "' " + _("no encontrado o no válido.") if oe.nil?
    end
    return resultado
  end

  # Importa una hoja de actividades para la matriz
  def importa_matriz_actividades hoja
    # El orden de las columnas es:
    # simples ->        resultado.codigo
    # multiples ->      codigo | descripcion 
    columnas = [ 'resultado.codigo', 'codigo', 'descripcion' ]
    indice=Hash[*columnas.enum_with_index.to_a.flatten]

    resultado = nil
    # Itera cada fila 
    hoja.each_with_index do |fila,numero_fila|
      # Si existe alguno de los datos basicos, es una fila nueva => obtenemos el resultado 
      resultado = resultado_para_actividad indice, fila if (fila[indice['resultado.codigo']]) && numero_fila > 0
      if (resultado)
        actualiza_actividad resultado, indice, fila if fila[indice['codigo']] || fila[indice['descripcion']]
      end
    end
  end

  def resultado_para_actividad indice, fila
    resultado = @proyecto.resultado.find_by_codigo fila[indice['resultado.codigo']]
    if resultado.nil?
      importacion_error "<br>" + _("Se produjeron errores procesando hoja de Actividad")
      importacion_error _("Resultado") + " '" + (fila[indice['resultado.codigo']].to_s) + "' " + _("no encontrado o no válido.")
    end
    return resultado
  end

  def actualiza_actividad resultado, indice, fila
    if fila[indice['codigo']]
      actividad = resultado.actividad.find_by_codigo(fila[indice['codigo']]) || Actividad.new
      actividad.codigo = fila[indice['codigo']].to_s if actividad.id.nil?
      actividad.resultado_id = resultado.id if actividad.id.nil?
      actividad.descripcion = fila[indice['descripcion']].to_s
      actividad.proyecto_id = @proyecto.id
      actividad.save
      importacion_error actividad, _("Actividad") + " '" + fila[indice['codigo']].to_s + "' " + _("en Resultado con código") + " '" + resultado.codigo + "'" 
    else
      importacion_error _("Actividad debe tener un código en hoja de Actividad para Resultado con código") + " '" + fila[indice['resultado.codigo']].to_s + "'"
    end
  end

  def actualiza_hipotesis objeto, indice, fila
    if objeto.hipotesis.find_by_descripcion(fila[indice['hipotesis.descripcion']]).nil?
      hipotesis=Hipotesis.new
      hipotesis.descripcion = fila[indice['hipotesis.descripcion']].to_s
      objeto.hipotesis << hipotesis
      hipotesis.save
      importacion_error hipotesis, _("Hipotesis") + " '" + fila[indice['hipotesis.descripcion']].to_s + "' en " + objeto.class.name + " " + _("con código") + " '" + objeto.codigo + "'"
    end
  end

  def actualiza_indicador objeto, indice, fila
    # El indicador deberia tener un codigo, pero el modelo permite que no lo tenga...
    # de momento lo dejamos asi, pero tendremos que tocar las validaciones del modelo
    if fila[indice['iov.codigo']]
      indicador = objeto.indicador.find_by_codigo(fila[indice['iov.codigo']]) || Indicador.new
      indicador.codigo = fila[indice['iov.codigo']].to_s if indicador.id.nil?
      objeto.indicador << indicador if indicador.id.nil? 
      indicador.descripcion = fila[indice['iov.descripcion']].to_s
      indicador.save
      importacion_error indicador, _("Indicador") + " '" + fila[indice['iov.codigo']].to_s + "' en " + objeto.class.name + " " + _("con código") + " '" + objeto.codigo + "'"
    else
      importacion_error _("En ") + objeto.class.name + " " + _("con código") + " " + objeto.codigo + " " + _("hay un Indicador que debería tener código.")
    end
  end

  def actualiza_fuente objeto, indice, fila
    indicador = objeto.indicador.find_by_codigo(fila[indice['fv.indicador']]) if fila[indice['fv.indicador']]
    # El indicador deberia tener un codigo, pero el modelo permite que no lo tenga...
    # de momento lo dejamos asi, pero tendremos que tocar las validaciones del modelo
    if fila[indice['fv.codigo']]
      fv = objeto.fuente_verificacion.find_by_codigo(fila[indice['fv.codigo']]) || FuenteVerificacion.new
      fv.codigo = fila[indice['fv.codigo']].to_s if fv.id.nil?
      fv.indicador_id = indicador.id if indicador
      objeto.fuente_verificacion << fv if fv.id.nil?
      fv.descripcion = fila[indice['fv.descripcion']].to_s
      fv.save
      importacion_error fv, _("Fuente de Verificación") + " '" + fila[indice['fv.codigo']].to_s + "' en " + objeto.class.name + " " + _("con código") + " '" + objeto.codigo + "'"
      importacion_error _("Indicador") + " '" + fila[indice['fv.indicador']].to_s + "' " + _("relacionado a la Fuente de Verificación") + " '" + fila[indice['fv.codigo']].to_s + "' " + "' en " + objeto.class.name + " " + _("con código") + " '" + objeto.codigo + "' " + _("no encontrado o no válido.") if ( indicador.nil? && fila[indice['fv.indicador']] )
    else
      importacion_error _("En ") + objeto.class.name + " " + _("con código") + " " + objeto.codigo + " " + _("hay una Fuente de Verificación que debería tener código.")
    end
  end

  #--
  # METODOS GENERALES 
  #++

  private
    def numero numero
      return numero.to_f
    end  

    def fecha fecha
      # Averiguar porque le falla a Jaime
      #return Date.parse fecha
      return fecha
    end

    def importacion_error objeto, detalle=nil
      @import_error = "" unless @import_error
      if objeto.class == String
        @import_error << objeto + "<br>"
      else
        if !objeto.errors.empty?
          @import_error << "<br>" + _("Se produjeron errores procesando") + " " + (detalle ? detalle.to_s : objeto.class.name) + ":<br>"
          objeto.errors.each {|a, m| @import_error += m + "<br>" }
        end
      end
    end

end
