#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de la entidad presupuesto. Este controlador es utilizado desde las secciones:
# * Sección agentes: se utiliza para la gestión del presupuesto relacionado con agente.
#

class PresupuestoAgentesController < ApplicationController

  # --
  # METODOS DE ENTRADA A PRESUPUESTO. Seleccion de ETAPA
  # ++

  # en agentes: se redirecciona por defecto a listado
  def index
    redirect_to :action => :filtrado_ordenado_iniciales
  end

  def filtrado_ordenado_iniciales
    if @agente.etapa.empty? 
      flash[:error] = _("Tiene que definir por lo menos una etapa")
      redirect_to :controller => :datos_agente, :action => :etapas
    else
      session[:presupuesto_agentes_asc_desc] = "ASC"
      session[:presupuesto_agentes_cadena_orden] = "fecha_inicio"
      session[:presupuesto_agentes_filtro] = @agente.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.id
      redirect_to :action => :listado
    end
  end

  def ordenado
    session[:presupuesto_agentes_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"
    session[:presupuesto_agentes_cadena_orden] = session[:presupuesto_agentes_orden] = params[:orden] ? params[:orden] : "fecha"
    session[:presupuesto_agentes_cadena_orden] = "partida.codigo" if params[:orden] == "partida.codigo_nombre"
    redirect_to :action => "listado"
  end

  def filtrado
    session[:presupuesto_agentes_filtro] = params[:filtro][:etapa]
    redirect_to :action => :listado
  end

  # --
  # METODOS DE GESTION DE Presupuesto
  # ++

  # en agentes: listado de presupuestos para el agente de la sessión
  def listado
    @etapa = Etapa.find( session[:presupuesto_agentes_filtro] )
    condiciones = ["presupuesto.agente_id = ? AND ( presupuesto.fecha_inicio BETWEEN ? AND ? OR presupuesto.fecha_fin BETWEEN ? AND ? )", @agente.id, @etapa.fecha_inicio, @etapa.fecha_fin, @etapa.fecha_inicio, @etapa.fecha_fin ]
    @presupuestos =  Presupuesto.paginate( :all, :order => session[:presupuesto_agentes_cadena_orden],
                                           :include => [:partida, :presupuesto_x_agente, :libro, :subpartida],
                                           :conditions => condiciones,
                                           :page => params[:page], :per_page => session[:presupuesto_numero] )
    @filtros = @agente.etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] } 
  end

  # en agentes: prepara el formulario de edición o creación de presupuesto
  def editar_nuevo
    @presupuesto = params[:id] ?  Presupuesto.find(params[:id]) : Presupuesto.new
    @etapa = Etapa.find( session[:presupuesto_agentes_filtro] )
    # SRAM 2011/05/16 - No tenemos porque buscar todos, solo aquellos que tengan al agente como implantador
    #@proyectos = Proyecto.find(:all).collect {|a| [a.nombre, a.id]}
    @proyectos = ProyectoXImplementador.find(:all, :conditions => { :agente_id => @agente.id }).collect { |pxi| [pxi.proyecto.nombre, pxi.proyecto_id] }
    @partidas = Partida.find(:all).collect {|a| [a.codigo_nombre, a.id]}
    @implementadores = [[@agente.nombre, @agente.id]]
    @monedas = Moneda.find(:all).collect {|a| [a.abreviatura, a.id]}
    render :partial => "formulario"
  end

  # en agentes: modifica o crea un presupuesto
  def modificar_crear
    @presupuesto = params[:id] ?  Presupuesto.find(params[:id]) : Presupuesto.new     # Asignamos los valores al presupueseto sin guadarlos para poder comprobar_fechas_etapa
    @etapa = Etapa.find( session[:presupuesto_agentes_filtro] )
    @presupuesto.attributes = params[:presupuesto]
    @presupuesto.subpartida.save if @presupuesto.subpartida
    (@presupuesto.save if @presupuesto.comprobar_fechas_etapa(Etapa.find(params[:etapa][:id]))  unless params[:etapa][:id] == "") unless params[:etapa] == nil
    @presupuesto.errors.add( _("fechas"), "Debe seleccionar etapa para comprobar fechas")  if params[:etapa]== nil or params[:etapa][:id] == ""
    flash[:objeto_modificado] = {:errors => @presupuesto.errors , :id => @presupuesto.id } 
    redirect_to :action => "listado"
  end

  # en agentes: elimina un presupuesto
  def eliminar
    @presupuesto = Presupuesto.find(params[:id])
    @presupuesto.destroy
    flash[:error] = @presupuesto 
    redirect_to :action => 'listado'
  end

  def etapas_proyecto
     @etapas_proyecto = Proyecto.find(params[:id]).etapa.collect {|e| [e.nombre + " ( " + e.fecha_inicio.to_s + "  "+ e.fecha_fin.to_s + " ) ", e.id]}
     render :inline =>  "<%= selector _('Selecciona una etapa'), 'etapa', 'id', @etapas_proyecto, false ,'2'  %>"
  end


    def auto_complete_for_presupuesto_subpartida_nombre
    @subpartidas = Subpartida.find(:all, :conditions => ['nombre like ?', "%#{params[:search]}%"])
    render :partial => "presupuesto_proyectos/auto_complete_for_presupuesto_subpartida_nombre"
  end
end

