#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de los resumenes de proyecto. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para presentar un resumen de proyecto según diversos parámetros

class ResumenProyectoController < ApplicationController

  before_filter :criterios_gasto, :only => [:gasto]
  before_filter :criterios_presupuesto, :only => [:presupuesto]

  def criterios_gasto
    if params[:financiador_implementador] == "" 
      flash[:error] = _("Debe seleccionar agente. Para ello primero seleccione tipo implementador o financiador.")
      redirect_to :action => "gasto"
    end
  end

  def criterios_presupuesto
    if params[:selector] and (params[:selector][:etapa].nil? or params[:selector][:etapa]=="" )
      flash[:error] = _("Debe seleccionar etapa. Si no existe ninguna debe dar de alta para este proyecto.")
      redirect_to :action => "presupuesto"
    end
  end


        # en proyectos: se redirecciona por defecto a listado_etapas
  def index
    redirect_to :action => 'presupuesto'
  end

        # en proyectos: selecciona la moneda y llama a estado_presupuesto si ya hay moneda y actividad/partida
  def presupuesto
    @listado_moneda = @proyecto.moneda.collect{|e| [e.nombre, e.id]} + [["Todas (con tasa aplicada)", "todas"]]
    @listado_etapa = @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    # Hacemos la busqueda si se seleccionan los criterios
    estado_presupuesto if params[:selector] and params[:selector][:moneda] != '' and params[:selector][:partida_actividad] != '' and params[:selector][:etapa] != ''
  end

  def estado_presupuesto
    partidas = Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    actividades = @proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    implementadores = @proyecto.implementador.collect{|i| {"id" => i.id, "nombre" => i.nombre}}
    financiadores = @proyecto.financiador.collect{|i| {"id" => i.id, "nombre" => i.nombre}}
    partidas_proyecto = @proyecto.partida_financiacion.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    
    case  [params[:selector][:partida_actividad],  params[:selector][:financiador_implementador]]
      when ["partida", "implementador"] then
         @datos = VPresupuesto.partida_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         @filas, @columnas = partidas, implementadores
      when ["partida", "financiador"] then
         @datos = VPresupuesto.partida_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         @filas, @columnas = partidas, financiadores
      when ["actividad", "implementador"] then
         @datos = VPresupuesto.actividad_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         @filas, @columnas = actividades, implementadores
      when ["actividad", "financiador"] then
         @datos = VPresupuesto.actividad_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         @filas, @columnas = actividades, financiadores
      when ["partida_proyecto", "implementador"] then
         @datos = VPresupuesto.partida_proyecto_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         @filas, @columnas = partidas_proyecto, implementadores
      when ["partida_proyecto", "financiador"] then
         @datos = VPresupuesto.partida_proyecto_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         @filas, @columnas = partidas_proyecto, financiadores
    end       



  end

  # en proyectos: selecciona la moneda, los agentes
  # y llama a busqueda_presupuesto
  def gasto
    @listado_moneda = @proyecto.moneda.collect{|e| [e.nombre, e.id]} + [["Todas las monedas (con tasa aplicada)", "todas"]]
    @listado_agente = @proyecto.implementador.collect{|e| [e.nombre, e.id]} + [["Todos los agentes", "todos"]]
    # Hacemos la busqueda si se seleccionan los criterios
    estado_gasto unless params[:moneda].nil? or params[:partida_actividad].nil? or params[:financiador_implementador].nil? 
  end

  def agentes
    render :partial => "agentes"
  end


  def estado_gasto
    @fecha_de_inicio = fecha params["fecha_inicio"]
    @fecha_de_fin = fecha params["fecha_fin"]
    @filas = case params[:partida_actividad]
              when "partida" then Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
              when "actividad" then @proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
              when "partida_proyecto" then @proyecto.partida_financiacion.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
             end
    @columnas = [{"id" => "1", "nombre" => "Presupuesto"}, {"id" => "2", "nombre" => "Gastos"}]

    case  [params[:partida_actividad],  params[:financiador_implementador]]
      when ["partida", "implementador"] then
         presupuestos = VPresupuestoDetallado.partida_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["partida", "financiador"] then
         presupuestos = VPresupuestoDetallado.partida_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["actividad", "implementador"] then
         presupuestos = VPresupuestoDetallado.actividad_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.actividad_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}


      when ["actividad", "financiador"] then
         presupuestos = VPresupuestoDetallado.actividad_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.actividad_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}


     when ["partida_proyecto", "implementador"] then      
         presupuestos = VPresupuestoDetallado.partida_proyecto_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_proyecto_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["partida_proyecto", "financiador"] then
         presupuestos = VPresupuestoDetallado.partida_proyecto_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_proyecto_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

    end 
    @datos = presupuestos + gastos 
    
  end


	# en proyectos: selecciona la caja para sacar el informe de arqueo
  def arqueo_caja
    @listado_libros = eval( "@" + singularizar_seccion ).libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}
    @listado_etapa = @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    # Si hemos especificado un libro para ver el informe
    if params[:selector] and params[:selector][:libro] and params[:selector][:libro] != ''
      @libro = Libro.find(params[:selector][:libro])
      @columnas_nombre = [_("fecha"), _("concepto"), _("entradas"), _("salidas")]
      condiciones = { }
      if params[:selector][:etapa] != ''
        @etapa = Etapa.find(params[:selector][:etapa])
        condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin
      end
      # Obtiene todos los gastos de la cuenta en el periodo
      gastos = @libro.gasto.find :all, :conditions => condiciones
      # Obtiene todas las transferencias de la cuenta en el periodo
      condiciones[:proyecto_id] = @proyecto.id
      transferencias = @libro.transferencia.find :all, :conditions => condiciones
      # Genera el array de gastos y transferencias para presentar
      @filas = Array.new
      transferencias.each do |transferencia|
        @filas.push(:fecha => transferencia.fecha, :concepto => _(transferencia.tipo.capitalize) + ( transferencia.observaciones ? ' (' + transferencia.observaciones + ')' : ''), :entrada => (transferencia.importe if transferencia.entrante_saliente=="entrante"), :salida => (transferencia.importe if transferencia.entrante_saliente!="entrante"), :tipo => transferencia.tipo)
      end
      gastos.each do |gasto|
        gasto.pago.each { |pago| @filas.push(:fecha => pago.fecha, :concepto => _("Pago") + " " + (gasto.concepto ? gasto.concepto : ''), :salida => pago.importe_convertido, :tipo => "pago") }
        #@filas.push(:fecha => gasto.fecha, :concepto => gasto.concepto || "Gasto", :salida => gasto.importe)
      end
      # Ordena por fecha
      @filas.sort! { |x,y| x[:fecha] <=> y[:fecha] }
    end
    
    render :template => "comunes/arqueo_caja"
  end
end
#done

