#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de la tasas de cambio para financiacion tanto para gasto como para presupuesto. Este controlador es utilizado desde las seccion financiaciones
#

class TasaCambioController < ApplicationController
  
  before_filter :cargar_entidades_sesion, :except => [:inicio, :index, :seleccionar_criterios]
  before_filter :comprobar_moneda_base, :except => [:inicio, :index, :seleccionar_criterios]

  def cargar_entidades_sesion
     if session[:etapa_tasas]
      @etapa = Etapa.find( session[:etapa_tasas] )
     else
      flash[:error] = _("Debe dar de alta etapas para aplicar tasas de cambio")
      redirect_to :action => :index      
     end
  end 

  def comprobar_moneda_base
     unless @proyecto.moneda_id
      flash[:error] = _("El proyecto no tiene una moneda base. Avise a administración para que dicha informacion sea completada.")
      redirect_to :action => :index
     end
  end

	#-- 
	# SELECCION DE ETAPAS PARA TASAS DE CAMBIO
	# ++
  def index
    redirect_to :action => :inicio
  end

	# en proyectos: se redirecciona por defecto a listado.
  def inicio
    session[:etapa_tasas] = nil
    session[:tipo_tasas] = nil
    @etapas = @proyecto.etapa
  end

  def seleccionar_criterios
    session[:etapa_tasas] = params[:selector][:etapa]
    session[:tipo_tasas] = params[:selector][:tipo]    
    if session[:tipo_tasas] == "gasto"
      redirect_to :action => :tasas_gasto
    elsif session[:tipo_tasas] == "presupuesto"
      redirect_to :action => :tasas_presupuesto
    end
  end


	# --
	# METODOS DE GESTION DE TASAS DE CAMBIO PARA EL PRESUPUESTO
	# ++


	# en proyectos: lista el presupuesto para la etapa y el proyecto de la financiacion, y muestra el formulario para aplicar tasas de cambio.
  def tasas_presupuesto
    condiciones = {"proyecto_id" => @proyecto.id, "presupuesto.etapa_id" => @etapa.id}
    @presupuestos = Presupuesto.paginate :all, :order => "partida.codigo", :include => [:partida], :conditions => condiciones, :page => params[:page], :per_page => 100
  end

	# en proyectos: selecciona el presupuesto para la edición de su tasa de cambio
  def editar_tasas_presupuesto
    @monedas_proyecto = @proyecto.moneda.reject {|m| m.id == @proyecto.moneda_id}
    render :partial => "formulario_tasas_presupuesto"
  end


	# en proyectos: modifica las tasas de cambio para el presupuesto. El metodo se apoya en:
  # * tasa_moneda_base_presupuesto
  # * seleccionar_presupuestos
  def modificar_tasas_presupuesto
    params[:moneda].each do |clave, valor|
      presupuestos = Presupuesto.find :all, :conditions => {"proyecto_id" => @proyecto.id, "presupuesto.etapa_id" => @etapa.id, "presupuesto.moneda_id" => clave}
      presupuestos.each {|p| p.update_attribute(:tasa_cambio, 1/moneda_a_float(valor[:tasa_cambio]))}
    end
    flash[:mensaje] = flash[:mensaje] == "" ? _("Se guardar correctamente los tasas de cambio aplicada a presupuesto") : flash[:mensaje]
    redirect_to :action => "tasas_presupuesto"
  end


	# --
	# METODOS DE GESTION DE TASAS DE CAMBIO PARA EL GASTO
	# ++
	# en proyectos: selecciona y guarda en la session el proyecto de la financiacion para el que vamos a aplicar las tasas de cambio al gasto.

	# en proyectos: lista los gastos para la etapa y el proyecto de la financiacion, y muestra el formulario para aplicar tasas de cambio.
  def tasas_gasto
    condiciones = {"gasto_x_proyecto.proyecto_id" => @proyecto.id, "gasto.fecha" => @etapa.fecha_inicio..@etapa.fecha_fin}
    @gastos = Gasto.paginate :all, :order => "fecha", :include => [:partida, :gasto_x_proyecto], :conditions => condiciones, :page => params[:page], :per_page => 100
  end

	# en proyectos: seleccionar los gastso a los que vamos a aplicar tasas de cambio.
  def seleccionar_gastos fecha_inicio, fecha_fin, moneda, numero

    condiciones["gasto.moneda_id"] = moneda if moneda

  end

	# en proyectos: 
  def editar_tasas_gasto
    @monedas = @proyecto.moneda.collect {|m| [m.abreviatura, m.id]}
    render :partial => "formulario_tasas_gasto"
  end

	# en proyectos: modifica las tasas de cambio para el gasto. El metodo se apoya en:
  # * seleccionar_gastos
  def modificar_tasas_gasto
    etapa = Etapa.find_by_id params[:etapa][:id]

    # Si elegimos toda la etapa, pillamos sus fechas
    if params[:selector][:etapa] == "1"
      fecha_inicio = etapa.fecha_inicio
      fecha_fin = etapa.fecha_fin
    else
      fecha_inicio = params[:intervalo][:fecha_inicio]
      fecha_fin = params[:intervalo][:fecha_fin]
    end

    # Dependiendo del tipo de aplicacion de tasa, actuamos
    if params[:moneda][:id].to_i == @proyecto.moneda_id
      tasa_seleccionada = 1.0 
    elsif params[:selector][:tipo] == "total"
      tasa_seleccionada = calcula_tasa_todas(params[:moneda][:id].to_i, fecha_inicio, fecha_fin)
    elsif params[:selector][:tipo] == "fifo"
      tasa_seleccionada = calcula_tasa_fifo(params[:moneda][:id].to_i, fecha_inicio, fecha_fin)
    else
      tasa_seleccionada = moneda_a_float(params[:cambio][:tasa])
    end

    tasa_seleccionada = 1/tasa_seleccionada if tasa_seleccionada

    gastos = Gasto.find :all,:include => :gasto_x_proyecto, :conditions => {"gasto_x_proyecto.proyecto_id" => @proyecto.id, "gasto.fecha" => fecha_inicio..fecha_fin, "gasto.moneda_id" => params[:moneda][:id]}
    borrar_tasas_cambio gastos
    gastos.each do |gasto|
      tasa_cambio = TasaCambioGasto.create(:gasto_id => gasto.id, :proyecto_id => @proyecto.id, :tasa_cambio => tasa_seleccionada) unless params[:selector][:tipo] == "fifo"
      tasa_cambio = TasaCambioGasto.create(:gasto_id => gasto.id, :proyecto_id => @proyecto.id, :tasa_cambio => tasa_seleccionada) if params[:selector][:tipo] == "fifo"
      flash[:mensaje] = tasa_cambio.errors.full_messages.inject(''){|todos, m| todos + m + "<br>" }
    end 
    flash[:mensaje] = flash[:mensaje] == "" ? _("Se guardar correctamente los tasas de cambio aplicada a gasto") : flash[:mensaje]
    flash[:mensaje] = _("No hay gastos en las fechas seleccionadas.") if gastos.empty?
    redirect_to :action => "tasas_gasto"
  end
 
	# en proyectos: borra las tasas de cambio para un determinado listado de objetos, sean presupuestos o gastos. 
  def borrar_tasas_cambio objetos
     for objeto in objetos
      tasa_cambio_objeto = objeto.tasa_cambio(@proyecto.id)
      tasa_cambio_objeto.destroy if tasa_cambio_objeto
     end
  end

  private

	# Pasamos de coma a punto
    def moneda_a_float cadena
      if cadena
        # primero comprobamos que el punto corresponde a miles ( formato => n.nnn,n )
        # y solo en ese caso lo eliminamos => Permitimos especificar decimales con punto
        numero = (cadena =~ /[\d]+\.[\d]{3},[\d]/) ? cadena.delete(".") : cadena
        # Si se esta usando la coma decimal, la cambiamos por un punto ( formato => n,nn o n,n )
        # REVISAR: Esto permite que haya coma para miles, pero impide que metan mas de 2 decimales con coma
        numero = numero.sub(/,/,".") if numero =~ /[\d]+,[\d]+$/
        return numero.to_f
      end
    end

	# Hace el calculo de tasa de cambio como promedio de todas las transferencias en un intervalo 
    def calcula_tasa_todas moneda_id, fecha_inicio, fecha_fin
      moneda_proyecto_id = @proyecto.moneda_id
      importe_total = tasa_temporal = 0
      #transferencias = @proyecto.transferencia.find :all, :conditions => { "fecha" => fecha_inicio..fecha_fin }
      transferencias = @proyecto.transferencia.find :all, :conditions => { "moneda_id" => moneda_proyecto_id, "entrante_saliente" => "saliente", "fecha" => fecha_inicio..fecha_fin }
      transferencias.each do |transferencia|
        if ((transferencia.moneda_id == moneda_proyecto_id && transferencia.libro_receptor_emisor.moneda_id == moneda_id && transferencia.entrante_saliente == "saliente") ||
	    (transferencia.moneda_id == moneda_id && transferencia.libro_receptor_emisor.moneda_id == moneda_proyecto_id && transferencia.entrante_saliente == "entrante") )
          importe_total += transferencia.importe
          tasa_temporal += transferencia.importe * transferencia.tasa_cambio
        end
      end
      return (tasa_temporal != 0 && importe_total != 0) ? tasa_temporal/importe_total : nil
    end

	# Hace el calculo de tasa de cambio segun se vaya gastando 
    def calcula_tasa_fifo moneda_id, fecha_inicio, fecha_fin
      return 3
    end
end
