#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de Transferencias. Este controlador es utilizado desde las secciones:
# * Sección agentes: se utiliza para gestionar las transferencias de un determinado agente
# * Sección proyectos: se utiliza para gestionar las transferencias de un proyecto

class TransferenciaController < ApplicationController
  
  
  # en proyectos y en agente: se redirecciona por defecto a filtrado_ordenado_iniciales creando el mensaje en funcion de la sección.
  def index
    objeto = singularizar_seccion
    redirect_to :action => :filtrado_ordenado_iniciales
  end

  # en proyectos y en agente: inicializa los defectos para ordenado y filtro y redirecciona a listado
  def filtrado_ordenado_iniciales
    if eval( "@" + singularizar_seccion ).etapa.empty? 
      flash[:error] = _("Tiene que definir por lo menos una etapa para acceder a la gestión de transferencias.")
      redirect_to :controller => :datos_proyecto, :action => :etapas
    else
      session[("transferencia_" + params[:seccion] + "_asc_desc").to_sym] ||= "ASC" 
      session[("transferencia_" + params[:seccion] + "_cadena_orden").to_sym] ||= "fecha"
      session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] ||= "todas"
      session[("transferencia_" + params[:seccion] + "_filtro_libro").to_sym] ||= "todas"
      session[("transferencia_" + params[:seccion] + "_filtro_tipo").to_sym] ||= "todos"
      redirect_to :action => :listado
    end
  end

  # en proyectos y agentes: establece los parametros de ordenación
  def ordenado
    session[("transferencia_" + params[:seccion] + "_asc_desc").to_sym] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[("transferencia_" + params[:seccion] + "_cadena_orden").to_sym] = session[("transferencia_" + params[:seccion] + "_orden").to_sym] = params[:orden] ? params[:orden] : "fecha"
    redirect_to :action => "listado"
  end

  # en proyectos y agentes: establece los parametros de filtro
  def filtrado
    session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] = params[:filtro][:etapa]
    session[("transferencia_" + params[:seccion] + "_filtro_tipo").to_sym] = params[:filtro][:tipo]
    session[("transferencia_" + params[:seccion] + "_filtro_libro").to_sym] = params[:filtro][:libro]
    redirect_to :action => :listado
  end

  # en proyectos y en agente: lista
  def listado
    @filtro_etapa = [["Todas","todas"]] +   eval( "@" + singularizar_seccion ).etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    @filtro_libro = [["Todas","todas"]] +   eval( "@" + singularizar_seccion ).libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]} 
    @filtro_tipo = filtro_tipo

    @libro = Libro.find( session[("transferencia_" + params[:seccion] + "_filtro_libro").to_sym] ) unless  session[("transferencia_" + params[:seccion] + "_filtro_libro").to_sym] == "todas"
    @etapa = Etapa.find( session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] ) unless  session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] == "todas"
    condiciones = { "libro.agente_id" => @agente.id } if params[:seccion] == "agentes"
    condiciones = { "proyecto_id" => @proyecto.id } if params[:seccion] == "proyectos"

    tipo_transferencia=session[("transferencia_" + params[:seccion] + "_filtro_tipo").to_sym].to_s
    condiciones[:tipo] = tipo_transferencia if tipo_transferencia != "todos" 
    condiciones[:libro_id] = @libro.id if @libro 
    condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin unless  session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] == "todas"
    @transferencias = Transferencia.paginate( :all, :conditions => condiciones,
                           :include => :libro,
                           :order => (session[("transferencia_" + params[:seccion] + "_cadena_orden").to_sym] + " " + session[("transferencia_" + params[:seccion] + "_asc_desc").to_sym] ),
                           :page => params[:page], :per_page => session[:transferencia_numero] )
  end


	# en proyectos y en agente: prepara el formulario de edición o creación
  def editar_nuevo
    @transferencia = params[:id] ?  Transferencia.find(params[:id]) : Transferencia.new
    @etapa = Etapa.find( session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] ) unless  session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] == "todas"
    @proyectos = @agente.proyecto_implementador.collect{ |p| [p.nombre, p.id] } if params[:seccion] == "agentes"
    @proyecto_asociado = @proyecto ? @proyecto : @transferencia.proyecto
    # Si proviene del formulario general y se ha elegido un tipo, pinta el formulario
    if params[:tipo]
      render :update do |page|
        page.replace "MB_caption", :inline => "<div id='MB_caption'>" + ( @proyecto ? "Proyecto: " + @proyecto.nombre : "Agente: " + @agente.nombre) + "<br>Nuevo movimiento: " + params[:tipo].capitalize + "</div>"
        page.replace params[:update], :partial => "formulario_" + params[:tipo]
        page.call("Modalbox.resizeToContent");
      end 
    # Si es uno nuevo, o ya existia el movimiento pinta el formulario correspondiente
    else
      tipos = filtro_tipo
      tipos.shift
      render :partial => "formulario" + ( @transferencia.id.nil? ? "" : "_" + @transferencia.tipo ), :locals => {:tipos => tipos}
    end
  end

  # en proyectos y en agente: modifica o crea
  def modificar_crear
    @etapa = Etapa.find( session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] ) unless  session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] == "todas"
    transferencia = params[:id] ?  Transferencia.find(params[:id]) : Transferencia.new 
    transferencia.attributes = params[:transferencia]
    params[:transferencia][(singularizar_seccion + "_id").to_sym] = eval( "@" + singularizar_seccion ).id
    transferencia.update_attributes params[:transferencia] if (session[("transferencia_" + params[:seccion] + "_filtro_etapa").to_sym] == "todas" or transferencia.comprobar_fecha_etapa @etapa)
    flash[:error] = transferencia
    redirect_to :action => "ordenado"
  end

  # en proyectos y en agente: elimina
  def eliminar
    Transferencia.find(params[:id]).destroy
    flash[:mensaje] = _('Se ha eliminado transferencia.') 
    redirect_to :action => 'ordenado'
  end

  # en proyectos y en agente: muestra la moneda del libro seleccionado
  def moneda_libro
    @moneda = params[:id] != "" ? Libro.find(params[:id]).moneda : nil
    render :inline => <<-FIN
      <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
      <%= hidden_field('transferencia', 'moneda_id', :value => @moneda.id) if @moneda%>
    FIN
  end

  # en proyectos y en agente: reacciona al cambio de libro
  def cambio_libro
    libro=Libro.find(params[:id]) if params[:id] != ""
    @moneda = libro ? libro.moneda : nil
    tipo_libro = libro.nil? ? "todos" : (libro.tipo == "banco" ? "caja \"chica\"" : "banco")
    @direccion = tipo_libro == "banco" ? params[:direccion][1] : params[:direccion][0]
    @proyecto_asociado = @proyecto ? @proyecto : @transferencia.proyecto
    render :update do |page|
      page.replace "moneda", :inline => <<-FIN
        <div class="elemento1_3" id="moneda">
          <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
          <%= hidden_field('transferencia', 'moneda_id', :value => @moneda.id) if @moneda %>
        </div>
      FIN
      page.replace "transferencia_entrante_saliente", :inline => <<-FIN
        <%= hidden_field('transferencia', 'entrante_saliente', :value => @direccion ) %>
      FIN
      page.replace "selector_segundo_libro", :partial => "selector_segundo_libro", :locals => { :tipo_libro => tipo_libro }
    end
  end

  def gastos
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      page.replace_html params[:update], :partial => "gastos", :locals => { :update => params[:update], :transferencia_id => params[:transferencia_id], :tipo => params[:tipo], :gastos => @gastos }
    end
  end

        # en proyectos y en financiación:  prepara el formulario de edición o creación de comentarios
  def asociar_gasto
    @gastos = @proyecto.gasto.collect{ |g| [g.importe.to_s + " -- " + g.partida.nombre + " -- " + (g.concepto || "") , g.id]}
    render :partial => "asociar_gasto", :locals => { :transferencia_id => params[:transferencia_id], :update => params[:update] }
  end

        # en proyectos y en financiación:  modifica o crea un comentario
  def guardar_asociar_gasto
    @gasto_x_transferencia = GastoXTransferencia.create( :gasto_id => params[:selector][:gasto], :transferencia_id => params[:transferencia_id] )
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      page.replace 'formulario', :inline => '<%= mensaje_error(@gasto_x_transferencia) %><br>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos, :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

        # en proyectos y en financiación:  elimina un comentario
  def eliminar_gasto
    @objeto = GastoXTransferencia.find(:first, :conditions => {:gasto_id => params[:gasto_id], :transferencia_id => params[:transferencia_id]})
    not @objeto.nil? and @objeto.destroy
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      #page.replace "gastos_" + params[:transferencia_id], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos }
      page.replace_html params[:update], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos, :update => params[:update]} 
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

  def cuenta_segunda_financiador
    @proyecto_asociado = Proyecto.find(params[:id])
    render :partial => "cuenta_segunda_financiador", :locals => { :filtro_tipo => params[:filtro_tipo] } 
  end

  private
    def filtro_tipo
      [ ["Todos","todos"], ["Transferencia","transferencia"],["Cambio","cambio"],["Cheque","cheque"],["Retirada","retirada"],["Adelanto","adelanto"],
                        ["Ingreso","ingreso"],["Intereses","intereses"],["Subvencion","subvencion"] ]
    end
end
