#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

module ResumenProyectoHelper


 #CARACTERES = { "4" => 128, "3" => 87, "5_2" => 74 ,  "2" => 64, "3_2" => 45, "1"  => 28, "3_4" => 21, "1_2" => 13, "1_3" => 11, "1_4"=> 10 , "1_8" => 5 }
  # Metodo que dibuja una tabla. Utiliza fila_tabla_resumen.
  # * datos: matriz con los datos en forma de array de hash con forma: {"importe", "columna_id", "fila_id"}
  # * columnas y filas: array de array con forma {"id", "nombre"} 
  # En "otros" (campos adicionales) puede incluirse: :columna_suma
  def tabla_resumen datos, columnas, filas, otros={}
    clases = otros[:clases] || ["","1","1_2_td"]
    cadena = '<div class = "tabla">'
    # pintamos la primera fila con los nombres de las columnas
    cadena << '<div class = "fila'+clases[0]+'"><b><div class="elemento'+clases[1]+'">&nbsp;</div>'
    cadena << columnas.inject('') {|suma, c| suma + '<div class="elemento'+ clases[2] +'">'+ c["nombre"] +'</div>' }
    # Incluimos columna de totales si los hubiese.
    cadena << '<div class="elemento'+ clases[2] +'">Totales</div>' if otros[:columna_suma]
    cadena << '<div class="elemento'+ clases[2] +'">Diferencia</div>' if otros[:columna_resta]
    cadena << '<div class="elemento'+ clases[2] +'">Desviacion %</div>' if otros[:columna_resta]
    cadena << '</b><div class = "linea"></div></div>'
    # recorremos fila a fila
    for fila in filas
       cadena << fila_tabla_resumen( datos, columnas, fila["nombre"], fila["id"], clases, otros )
    end
    cadena << fila_suma_columnas(datos, columnas, clases) if otros[:fila_suma]
    cadena << "</div>"
    return cadena
  end

  # Metodo que dibuja una fila de una tabla. Lo utiliza tabla resumen aunque puede utilizarse directamente.
  def fila_tabla_resumen datos, columnas, nombre, id, clases, otros={}
    fila = '<div class = "fila'+ clases[0] +'">'
    fila << '<div class="elemento'+ clases[1] +'">' + truncate(nombre, :lenght => caracteres(clases[1])) + '</div>'
    for columna in columnas
      fila << '<div class="elemento'+ clases[2] +'">' + valor_dato(datos, id, columna["id"]) + '</div>'
    end
    # Si el parametro columna adicional es "suma" sumamos los valores de todas las columnas de la fila en la ultima columna
    fila << '<div class="elemento'+ clases[2] +'">' + suma_fila(datos, id) + '</div>' if otros[:columna_suma] 
    fila << '<div class="elemento'+ clases[2] +'">' + number_with_delimiter(('%.2f' % resta_fila(datos, id) ).to_s , :separator => ",", :delimiter => ".")  + '</div>' if otros[:columna_resta]
    fila << '<div class="elemento'+ clases[2] +'">' + number_with_delimiter(('%.2f' % desviacion_fila(datos, id) ).to_s , :separator => ",", :delimiter => ".")  + '</div>' if otros[:desviacion] 
    fila << '<div class = "linea"></div></div>'
  end
  
  # Metodo que devuelve el valor de un dato dada una matriz de datos y el id de columna y de fila.
  def valor_dato datos, fila_id, columna_id
    valor = datos.detect {|v| v["columna_id"] == columna_id.to_s and v["fila_id"] == fila_id.to_s}
    return  valor ? number_with_delimiter(('%.2f' % valor["importe"] ).to_s , :separator => ",", :delimiter => ".") : "0,00"
  end

  def suma_fila datos, id
    total = datos.inject(0){|suma,d|  d["fila_id"] == id.to_s ? suma + d["importe"].to_f : suma }
    return total ? number_with_delimiter(('%.2f' % total ).to_s , :separator => ",", :delimiter => ".") : "0,00"
  end

  def resta_fila datos, id
    uno = (datos.detect {|v| v["columna_id"] == "1" and v["fila_id"] == id.to_s} || {"importe" => "0"} ).fetch("importe") 
    dos = (datos.detect {|v| v["columna_id"] == "2" and v["fila_id"] == id.to_s} || {"importe" => "0"} ).fetch("importe")
    total = dos.to_f - uno.to_f
    return total ?  total : "0"
  end 

  def desviacion_fila datos, id
    presupuesto = (datos.detect {|v| v["columna_id"] == "1" and v["fila_id"] == id.to_s} || {"importe" => "0"} ).fetch("importe").to_f
    if presupuesto and presupuesto != 0.to_f
      calculo = ( resta_fila(datos, id).to_f / presupuesto ) * 100
    else
      calculo = "0"
    end
    return calculo
  end 

    
  def fila_suma_columnas datos, columnas, clases   
    cadena = '<div class = "fila'+clases[0]+'"><div class="elemento'+clases[1]+'">TOTALES</div>'
    for columna in columnas
      cadena << '<div class="elemento'+ clases[2] +'">'
      total = datos.inject(0){|suma,d|  (d["columna_id"] == columna["id"].to_s and d["fila_id"] != nil) ? suma + d["importe"].to_f : suma} 
      cadena << (total ? number_with_delimiter(('%.2f' % total ).to_s , :separator => ",", :delimiter => ".") : "&nbsp;")
      cadena << '</div>'
    end
    cadena << '<div class = "linea"></div></div>'  
    return cadena  
  end


end
