#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# agente

class Agente < ActiveRecord::Base
  has_many :libro
  has_many :usuario_x_agente, :dependent => :destroy
  has_many :usuario, :through => :usuario_x_agente
  has_many :gasto_x_agente, :dependent => :destroy
  has_many :presupuesto_x_agente, :dependent => :destroy
  has_many :etapa, :dependent => :destroy
  validate :nombre_mayusculas
  validates_uniqueness_of :nombre, :message => _("Nombre") +   _(" repetido.")
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  has_and_belongs_to_many :proyecto_implementador, :join_table => "proyecto_x_implementador", :class_name => "Proyecto"
  has_and_belongs_to_many :proyecto_financiador, :join_table => "proyecto_x_financiador", :class_name => "Proyecto"
  before_destroy :verificar_borrado

  def nombre_mayusculas
    self.nombre = self.nombre.upcase
  end

  private

  def verificar_borrado
    # la relacion habtm no tiene :dependent por eso tenemos que borrar las relaciones de financiador y implementador 
    errors.add( _("esta relacionado con proyectos como implementador") ) unless self.proyecto_implementador.each do |proyecto|
      self.proyecto_implementador.delete(proyecto)
    end
    errors.add( _("esta relacionado con proyectos como financiador") ) unless self.proyecto_financiador.each do |proyecto|
      self.proyecto_financiador.delete(proyecto)
    end
    errors.add( "presupuesto",_("hay presupuestos")) unless self.presupuesto_x_agente.empty?
    errors.add( "gasto", _("hay gastos")) unless self.gasto_x_agente.empty?
    errors.add( "libro", _("hay libros")) unless self.libro.empty?
    errors.add_to_base( _("Un agente tiene que estar vacio para poder ser borrado.") ) unless errors.empty?
    return false unless errors.empty?
  end
  
end
  

