#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# GastoXProyecto: relaciona gasto con proyecto y el porcentaje de esta relación

class GastoXProyecto < ActiveRecord::Base

  untranslate_all

  # acts_as_reportable
  belongs_to :gasto
  belongs_to :marcado

  validates_presence_of :proyecto_id, :message => _("Proyecto") + _(" no puede estar vacío.")
  validates_uniqueness_of :proyecto_id, :scope => :gasto_id, :message => _("Proyecto") + _(" repetido.")
  validates_numericality_of :porcentaje, :less_than_or_equal_to => 1, :message => _("Pocentaje") + _(" debe estar entre 0 y 1")
  validates_numericality_of :porcentaje, :greater_than => 0, :message => _("Proyecto") + _(" con porcentaje 0 no se guarda")

  # Proyecto de la relación.
  def proyecto
    return Proyecto.find self.proyecto_id
  end

  # Importe * el porcentaje asignado en la relacion
  def importe
    return (self.gasto.importe * self.porcentaje)
  end

  # Porcentaje en formato para la visualización
  def porcentaje_to_s
    return (self.porcentaje * 100).to_i.to_s + "%"
  end

end
